#-*- coding: utf-8 -*-
# Generated from catalan.sbl by Snowball 3.0.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class CatalanStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from catalan.sbl by Snowball 3.0.1 - https://snowballstem.org/
    '''

    g_v = {u"a", u"e", u"i", u"o", u"u", u"á", u"à", u"é", u"è", u"í", u"ï", u"ó", u"ò", u"ú", u"ü"}

    I_p2 = 0
    I_p1 = 0

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            if not self.go_out_grouping(CatalanStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(CatalanStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p1 = self.cursor
            if not self.go_out_grouping(CatalanStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(CatalanStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab0: pass
        self.cursor = v_1
        return True

    def __r_cleaning(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(CatalanStemmer.a_0)
                self.ket = self.cursor
                if among_var == 1:
                    if not self.slice_from(u"a"):
                        return False
                elif among_var == 2:
                    if not self.slice_from(u"e"):
                        return False
                elif among_var == 3:
                    if not self.slice_from(u"i"):
                        return False
                elif among_var == 4:
                    if not self.slice_from(u"o"):
                        return False
                elif among_var == 5:
                    if not self.slice_from(u"u"):
                        return False
                elif among_var == 6:
                    if not self.slice_from(u"."):
                        return False
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_R2(self):
        return self.I_p2 <= self.cursor

    def __r_attached_pronoun(self):
        self.ket = self.cursor
        if self.find_among_b(CatalanStemmer.a_1) == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if not self.slice_del():
            return False

        return True

    def __r_standard_suffix(self):
        self.ket = self.cursor
        among_var = self.find_among_b(CatalanStemmer.a_2)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R1():
                return False
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

        elif among_var == 3:
            if not self.__r_R2():
                return False
            if not self.slice_from(u"log"):
                return False
        elif among_var == 4:
            if not self.__r_R2():
                return False
            if not self.slice_from(u"ic"):
                return False
        else:
            if not self.__r_R1():
                return False
            if not self.slice_from(u"c"):
                return False
        return True

    def __r_verb_suffix(self):
        self.ket = self.cursor
        among_var = self.find_among_b(CatalanStemmer.a_3)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R1():
                return False
            if not self.slice_del():
                return False

        else:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

        return True

    def __r_residual_suffix(self):
        self.ket = self.cursor
        among_var = self.find_among_b(CatalanStemmer.a_4)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R1():
                return False
            if not self.slice_del():
                return False

        else:
            if not self.__r_R1():
                return False
            if not self.slice_from(u"ic"):
                return False
        return True

    def _stem(self):
        self.__r_mark_regions()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_2 = self.limit - self.cursor
        self.__r_attached_pronoun()
        self.cursor = self.limit - v_2
        v_3 = self.limit - self.cursor
        try:
            try:
                v_4 = self.limit - self.cursor
                try:
                    if not self.__r_standard_suffix():
                        raise lab2()
                    raise lab1()
                except lab2: pass
                self.cursor = self.limit - v_4
                if not self.__r_verb_suffix():
                    raise lab0()
            except lab1: pass
        except lab0: pass
        self.cursor = self.limit - v_3
        v_5 = self.limit - self.cursor
        self.__r_residual_suffix()
        self.cursor = self.limit - v_5
        self.cursor = self.limit_backward
        v_6 = self.cursor
        self.__r_cleaning()
        self.cursor = v_6
        return True

    a_0 = [
        Among(u"", -1, 7),
        Among(u"·", 0, 6),
        Among(u"à", 0, 1),
        Among(u"á", 0, 1),
        Among(u"è", 0, 2),
        Among(u"é", 0, 2),
        Among(u"ì", 0, 3),
        Among(u"í", 0, 3),
        Among(u"ï", 0, 3),
        Among(u"ò", 0, 4),
        Among(u"ó", 0, 4),
        Among(u"ú", 0, 5),
        Among(u"ü", 0, 5)
    ]

    a_1 = [
        Among(u"la", -1, 1),
        Among(u"-la", 0, 1),
        Among(u"sela", 0, 1),
        Among(u"le", -1, 1),
        Among(u"me", -1, 1),
        Among(u"-me", 4, 1),
        Among(u"se", -1, 1),
        Among(u"-te", -1, 1),
        Among(u"hi", -1, 1),
        Among(u"'hi", 8, 1),
        Among(u"li", -1, 1),
        Among(u"-li", 10, 1),
        Among(u"'l", -1, 1),
        Among(u"'m", -1, 1),
        Among(u"-m", -1, 1),
        Among(u"'n", -1, 1),
        Among(u"-n", -1, 1),
        Among(u"ho", -1, 1),
        Among(u"'ho", 17, 1),
        Among(u"lo", -1, 1),
        Among(u"selo", 19, 1),
        Among(u"'s", -1, 1),
        Among(u"las", -1, 1),
        Among(u"selas", 22, 1),
        Among(u"les", -1, 1),
        Among(u"-les", 24, 1),
        Among(u"'ls", -1, 1),
        Among(u"-ls", -1, 1),
        Among(u"'ns", -1, 1),
        Among(u"-ns", -1, 1),
        Among(u"ens", -1, 1),
        Among(u"los", -1, 1),
        Among(u"selos", 31, 1),
        Among(u"nos", -1, 1),
        Among(u"-nos", 33, 1),
        Among(u"vos", -1, 1),
        Among(u"us", -1, 1),
        Among(u"-us", 36, 1),
        Among(u"'t", -1, 1)
    ]

    a_2 = [
        Among(u"ica", -1, 4),
        Among(u"lógica", 0, 3),
        Among(u"enca", -1, 1),
        Among(u"ada", -1, 2),
        Among(u"ancia", -1, 1),
        Among(u"encia", -1, 1),
        Among(u"ència", -1, 1),
        Among(u"ícia", -1, 1),
        Among(u"logia", -1, 3),
        Among(u"inia", -1, 1),
        Among(u"íinia", 9, 1),
        Among(u"eria", -1, 1),
        Among(u"ària", -1, 1),
        Among(u"atòria", -1, 1),
        Among(u"alla", -1, 1),
        Among(u"ella", -1, 1),
        Among(u"ívola", -1, 1),
        Among(u"ima", -1, 1),
        Among(u"íssima", 17, 1),
        Among(u"quíssima", 18, 5),
        Among(u"ana", -1, 1),
        Among(u"ina", -1, 1),
        Among(u"era", -1, 1),
        Among(u"sfera", 22, 1),
        Among(u"ora", -1, 1),
        Among(u"dora", 24, 1),
        Among(u"adora", 25, 1),
        Among(u"adura", -1, 1),
        Among(u"esa", -1, 1),
        Among(u"osa", -1, 1),
        Among(u"assa", -1, 1),
        Among(u"essa", -1, 1),
        Among(u"issa", -1, 1),
        Among(u"eta", -1, 1),
        Among(u"ita", -1, 1),
        Among(u"ota", -1, 1),
        Among(u"ista", -1, 1),
        Among(u"ialista", 36, 1),
        Among(u"ionista", 36, 1),
        Among(u"iva", -1, 1),
        Among(u"ativa", 39, 1),
        Among(u"nça", -1, 1),
        Among(u"logía", -1, 3),
        Among(u"ic", -1, 4),
        Among(u"ístic", 43, 1),
        Among(u"enc", -1, 1),
        Among(u"esc", -1, 1),
        Among(u"ud", -1, 1),
        Among(u"atge", -1, 1),
        Among(u"ble", -1, 1),
        Among(u"able", 49, 1),
        Among(u"ible", 49, 1),
        Among(u"isme", -1, 1),
        Among(u"ialisme", 52, 1),
        Among(u"ionisme", 52, 1),
        Among(u"ivisme", 52, 1),
        Among(u"aire", -1, 1),
        Among(u"icte", -1, 1),
        Among(u"iste", -1, 1),
        Among(u"ici", -1, 1),
        Among(u"íci", -1, 1),
        Among(u"logi", -1, 3),
        Among(u"ari", -1, 1),
        Among(u"tori", -1, 1),
        Among(u"al", -1, 1),
        Among(u"il", -1, 1),
        Among(u"all", -1, 1),
        Among(u"ell", -1, 1),
        Among(u"ívol", -1, 1),
        Among(u"isam", -1, 1),
        Among(u"issem", -1, 1),
        Among(u"ìssem", -1, 1),
        Among(u"íssem", -1, 1),
        Among(u"íssim", -1, 1),
        Among(u"quíssim", 73, 5),
        Among(u"amen", -1, 1),
        Among(u"ìssin", -1, 1),
        Among(u"ar", -1, 1),
        Among(u"ificar", 77, 1),
        Among(u"egar", 77, 1),
        Among(u"ejar", 77, 1),
        Among(u"itar", 77, 1),
        Among(u"itzar", 77, 1),
        Among(u"fer", -1, 1),
        Among(u"or", -1, 1),
        Among(u"dor", 84, 1),
        Among(u"dur", -1, 1),
        Among(u"doras", -1, 1),
        Among(u"ics", -1, 4),
        Among(u"lógics", 88, 3),
        Among(u"uds", -1, 1),
        Among(u"nces", -1, 1),
        Among(u"ades", -1, 2),
        Among(u"ancies", -1, 1),
        Among(u"encies", -1, 1),
        Among(u"ències", -1, 1),
        Among(u"ícies", -1, 1),
        Among(u"logies", -1, 3),
        Among(u"inies", -1, 1),
        Among(u"ínies", -1, 1),
        Among(u"eries", -1, 1),
        Among(u"àries", -1, 1),
        Among(u"atòries", -1, 1),
        Among(u"bles", -1, 1),
        Among(u"ables", 103, 1),
        Among(u"ibles", 103, 1),
        Among(u"imes", -1, 1),
        Among(u"íssimes", 106, 1),
        Among(u"quíssimes", 107, 5),
        Among(u"formes", -1, 1),
        Among(u"ismes", -1, 1),
        Among(u"ialismes", 110, 1),
        Among(u"ines", -1, 1),
        Among(u"eres", -1, 1),
        Among(u"ores", -1, 1),
        Among(u"dores", 114, 1),
        Among(u"idores", 115, 1),
        Among(u"dures", -1, 1),
        Among(u"eses", -1, 1),
        Among(u"oses", -1, 1),
        Among(u"asses", -1, 1),
        Among(u"ictes", -1, 1),
        Among(u"ites", -1, 1),
        Among(u"otes", -1, 1),
        Among(u"istes", -1, 1),
        Among(u"ialistes", 124, 1),
        Among(u"ionistes", 124, 1),
        Among(u"iques", -1, 4),
        Among(u"lógiques", 127, 3),
        Among(u"ives", -1, 1),
        Among(u"atives", 129, 1),
        Among(u"logíes", -1, 3),
        Among(u"allengües", -1, 1),
        Among(u"icis", -1, 1),
        Among(u"ícis", -1, 1),
        Among(u"logis", -1, 3),
        Among(u"aris", -1, 1),
        Among(u"toris", -1, 1),
        Among(u"ls", -1, 1),
        Among(u"als", 138, 1),
        Among(u"ells", 138, 1),
        Among(u"ims", -1, 1),
        Among(u"íssims", 141, 1),
        Among(u"quíssims", 142, 5),
        Among(u"ions", -1, 1),
        Among(u"cions", 144, 1),
        Among(u"acions", 145, 2),
        Among(u"esos", -1, 1),
        Among(u"osos", -1, 1),
        Among(u"assos", -1, 1),
        Among(u"issos", -1, 1),
        Among(u"ers", -1, 1),
        Among(u"ors", -1, 1),
        Among(u"dors", 152, 1),
        Among(u"adors", 153, 1),
        Among(u"idors", 153, 1),
        Among(u"ats", -1, 1),
        Among(u"itats", 156, 1),
        Among(u"bilitats", 157, 1),
        Among(u"ivitats", 157, 1),
        Among(u"ativitats", 159, 1),
        Among(u"ïtats", 156, 1),
        Among(u"ets", -1, 1),
        Among(u"ants", -1, 1),
        Among(u"ents", -1, 1),
        Among(u"ments", 164, 1),
        Among(u"aments", 165, 1),
        Among(u"ots", -1, 1),
        Among(u"uts", -1, 1),
        Among(u"ius", -1, 1),
        Among(u"trius", 169, 1),
        Among(u"atius", 169, 1),
        Among(u"ès", -1, 1),
        Among(u"és", -1, 1),
        Among(u"ís", -1, 1),
        Among(u"dís", 174, 1),
        Among(u"ós", -1, 1),
        Among(u"itat", -1, 1),
        Among(u"bilitat", 177, 1),
        Among(u"ivitat", 177, 1),
        Among(u"ativitat", 179, 1),
        Among(u"ïtat", -1, 1),
        Among(u"et", -1, 1),
        Among(u"ant", -1, 1),
        Among(u"ent", -1, 1),
        Among(u"ient", 184, 1),
        Among(u"ment", 184, 1),
        Among(u"ament", 186, 1),
        Among(u"isament", 187, 1),
        Among(u"ot", -1, 1),
        Among(u"isseu", -1, 1),
        Among(u"ìsseu", -1, 1),
        Among(u"ísseu", -1, 1),
        Among(u"triu", -1, 1),
        Among(u"íssiu", -1, 1),
        Among(u"atiu", -1, 1),
        Among(u"ó", -1, 1),
        Among(u"ió", 196, 1),
        Among(u"ció", 197, 1),
        Among(u"ació", 198, 1)
    ]

    a_3 = [
        Among(u"aba", -1, 1),
        Among(u"esca", -1, 1),
        Among(u"isca", -1, 1),
        Among(u"ïsca", -1, 1),
        Among(u"ada", -1, 1),
        Among(u"ida", -1, 1),
        Among(u"uda", -1, 1),
        Among(u"ïda", -1, 1),
        Among(u"ia", -1, 1),
        Among(u"aria", 8, 1),
        Among(u"iria", 8, 1),
        Among(u"ara", -1, 1),
        Among(u"iera", -1, 1),
        Among(u"ira", -1, 1),
        Among(u"adora", -1, 1),
        Among(u"ïra", -1, 1),
        Among(u"ava", -1, 1),
        Among(u"ixa", -1, 1),
        Among(u"itza", -1, 1),
        Among(u"ía", -1, 1),
        Among(u"aría", 19, 1),
        Among(u"ería", 19, 1),
        Among(u"iría", 19, 1),
        Among(u"ïa", -1, 1),
        Among(u"isc", -1, 1),
        Among(u"ïsc", -1, 1),
        Among(u"ad", -1, 1),
        Among(u"ed", -1, 1),
        Among(u"id", -1, 1),
        Among(u"ie", -1, 1),
        Among(u"re", -1, 1),
        Among(u"dre", 30, 1),
        Among(u"ase", -1, 1),
        Among(u"iese", -1, 1),
        Among(u"aste", -1, 1),
        Among(u"iste", -1, 1),
        Among(u"ii", -1, 1),
        Among(u"ini", -1, 1),
        Among(u"esqui", -1, 1),
        Among(u"eixi", -1, 1),
        Among(u"itzi", -1, 1),
        Among(u"am", -1, 1),
        Among(u"em", -1, 1),
        Among(u"arem", 42, 1),
        Among(u"irem", 42, 1),
        Among(u"àrem", 42, 1),
        Among(u"írem", 42, 1),
        Among(u"àssem", 42, 1),
        Among(u"éssem", 42, 1),
        Among(u"iguem", 42, 1),
        Among(u"ïguem", 42, 1),
        Among(u"avem", 42, 1),
        Among(u"àvem", 42, 1),
        Among(u"ávem", 42, 1),
        Among(u"irìem", 42, 1),
        Among(u"íem", 42, 1),
        Among(u"aríem", 55, 1),
        Among(u"iríem", 55, 1),
        Among(u"assim", -1, 1),
        Among(u"essim", -1, 1),
        Among(u"issim", -1, 1),
        Among(u"àssim", -1, 1),
        Among(u"èssim", -1, 1),
        Among(u"éssim", -1, 1),
        Among(u"íssim", -1, 1),
        Among(u"ïm", -1, 1),
        Among(u"an", -1, 1),
        Among(u"aban", 66, 1),
        Among(u"arian", 66, 1),
        Among(u"aran", 66, 1),
        Among(u"ieran", 66, 1),
        Among(u"iran", 66, 1),
        Among(u"ían", 66, 1),
        Among(u"arían", 72, 1),
        Among(u"erían", 72, 1),
        Among(u"irían", 72, 1),
        Among(u"en", -1, 1),
        Among(u"ien", 76, 1),
        Among(u"arien", 77, 1),
        Among(u"irien", 77, 1),
        Among(u"aren", 76, 1),
        Among(u"eren", 76, 1),
        Among(u"iren", 76, 1),
        Among(u"àren", 76, 1),
        Among(u"ïren", 76, 1),
        Among(u"asen", 76, 1),
        Among(u"iesen", 76, 1),
        Among(u"assen", 76, 1),
        Among(u"essen", 76, 1),
        Among(u"issen", 76, 1),
        Among(u"éssen", 76, 1),
        Among(u"ïssen", 76, 1),
        Among(u"esquen", 76, 1),
        Among(u"isquen", 76, 1),
        Among(u"ïsquen", 76, 1),
        Among(u"aven", 76, 1),
        Among(u"ixen", 76, 1),
        Among(u"eixen", 96, 1),
        Among(u"ïxen", 76, 1),
        Among(u"ïen", 76, 1),
        Among(u"in", -1, 1),
        Among(u"inin", 100, 1),
        Among(u"sin", 100, 1),
        Among(u"isin", 102, 1),
        Among(u"assin", 102, 1),
        Among(u"essin", 102, 1),
        Among(u"issin", 102, 1),
        Among(u"ïssin", 102, 1),
        Among(u"esquin", 100, 1),
        Among(u"eixin", 100, 1),
        Among(u"aron", -1, 1),
        Among(u"ieron", -1, 1),
        Among(u"arán", -1, 1),
        Among(u"erán", -1, 1),
        Among(u"irán", -1, 1),
        Among(u"iïn", -1, 1),
        Among(u"ado", -1, 1),
        Among(u"ido", -1, 1),
        Among(u"ando", -1, 2),
        Among(u"iendo", -1, 1),
        Among(u"io", -1, 1),
        Among(u"ixo", -1, 1),
        Among(u"eixo", 121, 1),
        Among(u"ïxo", -1, 1),
        Among(u"itzo", -1, 1),
        Among(u"ar", -1, 1),
        Among(u"tzar", 125, 1),
        Among(u"er", -1, 1),
        Among(u"eixer", 127, 1),
        Among(u"ir", -1, 1),
        Among(u"ador", -1, 1),
        Among(u"as", -1, 1),
        Among(u"abas", 131, 1),
        Among(u"adas", 131, 1),
        Among(u"idas", 131, 1),
        Among(u"aras", 131, 1),
        Among(u"ieras", 131, 1),
        Among(u"ías", 131, 1),
        Among(u"arías", 137, 1),
        Among(u"erías", 137, 1),
        Among(u"irías", 137, 1),
        Among(u"ids", -1, 1),
        Among(u"es", -1, 1),
        Among(u"ades", 142, 1),
        Among(u"ides", 142, 1),
        Among(u"udes", 142, 1),
        Among(u"ïdes", 142, 1),
        Among(u"atges", 142, 1),
        Among(u"ies", 142, 1),
        Among(u"aries", 148, 1),
        Among(u"iries", 148, 1),
        Among(u"ares", 142, 1),
        Among(u"ires", 142, 1),
        Among(u"adores", 142, 1),
        Among(u"ïres", 142, 1),
        Among(u"ases", 142, 1),
        Among(u"ieses", 142, 1),
        Among(u"asses", 142, 1),
        Among(u"esses", 142, 1),
        Among(u"isses", 142, 1),
        Among(u"ïsses", 142, 1),
        Among(u"ques", 142, 1),
        Among(u"esques", 161, 1),
        Among(u"ïsques", 161, 1),
        Among(u"aves", 142, 1),
        Among(u"ixes", 142, 1),
        Among(u"eixes", 165, 1),
        Among(u"ïxes", 142, 1),
        Among(u"ïes", 142, 1),
        Among(u"abais", -1, 1),
        Among(u"arais", -1, 1),
        Among(u"ierais", -1, 1),
        Among(u"íais", -1, 1),
        Among(u"aríais", 172, 1),
        Among(u"eríais", 172, 1),
        Among(u"iríais", 172, 1),
        Among(u"aseis", -1, 1),
        Among(u"ieseis", -1, 1),
        Among(u"asteis", -1, 1),
        Among(u"isteis", -1, 1),
        Among(u"inis", -1, 1),
        Among(u"sis", -1, 1),
        Among(u"isis", 181, 1),
        Among(u"assis", 181, 1),
        Among(u"essis", 181, 1),
        Among(u"issis", 181, 1),
        Among(u"ïssis", 181, 1),
        Among(u"esquis", -1, 1),
        Among(u"eixis", -1, 1),
        Among(u"itzis", -1, 1),
        Among(u"áis", -1, 1),
        Among(u"aréis", -1, 1),
        Among(u"eréis", -1, 1),
        Among(u"iréis", -1, 1),
        Among(u"ams", -1, 1),
        Among(u"ados", -1, 1),
        Among(u"idos", -1, 1),
        Among(u"amos", -1, 1),
        Among(u"ábamos", 197, 1),
        Among(u"áramos", 197, 1),
        Among(u"iéramos", 197, 1),
        Among(u"íamos", 197, 1),
        Among(u"aríamos", 201, 1),
        Among(u"eríamos", 201, 1),
        Among(u"iríamos", 201, 1),
        Among(u"aremos", -1, 1),
        Among(u"eremos", -1, 1),
        Among(u"iremos", -1, 1),
        Among(u"ásemos", -1, 1),
        Among(u"iésemos", -1, 1),
        Among(u"imos", -1, 1),
        Among(u"adors", -1, 1),
        Among(u"ass", -1, 1),
        Among(u"erass", 212, 1),
        Among(u"ess", -1, 1),
        Among(u"ats", -1, 1),
        Among(u"its", -1, 1),
        Among(u"ents", -1, 1),
        Among(u"às", -1, 1),
        Among(u"aràs", 218, 1),
        Among(u"iràs", 218, 1),
        Among(u"arás", -1, 1),
        Among(u"erás", -1, 1),
        Among(u"irás", -1, 1),
        Among(u"és", -1, 1),
        Among(u"arés", 224, 1),
        Among(u"ís", -1, 1),
        Among(u"iïs", -1, 1),
        Among(u"at", -1, 1),
        Among(u"it", -1, 1),
        Among(u"ant", -1, 1),
        Among(u"ent", -1, 1),
        Among(u"int", -1, 1),
        Among(u"ut", -1, 1),
        Among(u"ït", -1, 1),
        Among(u"au", -1, 1),
        Among(u"erau", 235, 1),
        Among(u"ieu", -1, 1),
        Among(u"ineu", -1, 1),
        Among(u"areu", -1, 1),
        Among(u"ireu", -1, 1),
        Among(u"àreu", -1, 1),
        Among(u"íreu", -1, 1),
        Among(u"asseu", -1, 1),
        Among(u"esseu", -1, 1),
        Among(u"eresseu", 244, 1),
        Among(u"àsseu", -1, 1),
        Among(u"ésseu", -1, 1),
        Among(u"igueu", -1, 1),
        Among(u"ïgueu", -1, 1),
        Among(u"àveu", -1, 1),
        Among(u"áveu", -1, 1),
        Among(u"itzeu", -1, 1),
        Among(u"ìeu", -1, 1),
        Among(u"irìeu", 253, 1),
        Among(u"íeu", -1, 1),
        Among(u"aríeu", 255, 1),
        Among(u"iríeu", 255, 1),
        Among(u"assiu", -1, 1),
        Among(u"issiu", -1, 1),
        Among(u"àssiu", -1, 1),
        Among(u"èssiu", -1, 1),
        Among(u"éssiu", -1, 1),
        Among(u"íssiu", -1, 1),
        Among(u"ïu", -1, 1),
        Among(u"ix", -1, 1),
        Among(u"eix", 265, 1),
        Among(u"ïx", -1, 1),
        Among(u"itz", -1, 1),
        Among(u"ià", -1, 1),
        Among(u"arà", -1, 1),
        Among(u"irà", -1, 1),
        Among(u"itzà", -1, 1),
        Among(u"ará", -1, 1),
        Among(u"erá", -1, 1),
        Among(u"irá", -1, 1),
        Among(u"irè", -1, 1),
        Among(u"aré", -1, 1),
        Among(u"eré", -1, 1),
        Among(u"iré", -1, 1),
        Among(u"í", -1, 1),
        Among(u"iï", -1, 1),
        Among(u"ió", -1, 1)
    ]

    a_4 = [
        Among(u"a", -1, 1),
        Among(u"e", -1, 1),
        Among(u"i", -1, 1),
        Among(u"ïn", -1, 1),
        Among(u"o", -1, 1),
        Among(u"ir", -1, 1),
        Among(u"s", -1, 1),
        Among(u"is", 6, 1),
        Among(u"os", 6, 1),
        Among(u"ïs", 6, 1),
        Among(u"it", -1, 1),
        Among(u"eu", -1, 1),
        Among(u"iu", -1, 1),
        Among(u"iqu", -1, 2),
        Among(u"itz", -1, 1),
        Among(u"à", -1, 1),
        Among(u"á", -1, 1),
        Among(u"é", -1, 1),
        Among(u"ì", -1, 1),
        Among(u"í", -1, 1),
        Among(u"ï", -1, 1),
        Among(u"ó", -1, 1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass



#-*- coding: utf-8 -*-
# Generated from arabic.sbl by Snowball 3.0.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class ArabicStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from arabic.sbl by Snowball 3.0.1 - https://snowballstem.org/
    '''

    B_is_defined = False
    B_is_verb = False
    B_is_noun = False

    def __r_Normalize_pre(self):
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    try:
                        v_3 = self.cursor
                        try:
                            self.bra = self.cursor
                            among_var = self.find_among(ArabicStemmer.a_0)
                            if among_var == 0:
                                raise lab3()
                            self.ket = self.cursor
                            if among_var == 1:
                                if not self.slice_del():
                                    return False

                            elif among_var == 2:
                                if not self.slice_from(u"0"):
                                    return False
                            elif among_var == 3:
                                if not self.slice_from(u"1"):
                                    return False
                            elif among_var == 4:
                                if not self.slice_from(u"2"):
                                    return False
                            elif among_var == 5:
                                if not self.slice_from(u"3"):
                                    return False
                            elif among_var == 6:
                                if not self.slice_from(u"4"):
                                    return False
                            elif among_var == 7:
                                if not self.slice_from(u"5"):
                                    return False
                            elif among_var == 8:
                                if not self.slice_from(u"6"):
                                    return False
                            elif among_var == 9:
                                if not self.slice_from(u"7"):
                                    return False
                            elif among_var == 10:
                                if not self.slice_from(u"8"):
                                    return False
                            elif among_var == 11:
                                if not self.slice_from(u"9"):
                                    return False
                            elif among_var == 12:
                                if not self.slice_from(u"\u0621"):
                                    return False
                            elif among_var == 13:
                                if not self.slice_from(u"\u0623"):
                                    return False
                            elif among_var == 14:
                                if not self.slice_from(u"\u0625"):
                                    return False
                            elif among_var == 15:
                                if not self.slice_from(u"\u0626"):
                                    return False
                            elif among_var == 16:
                                if not self.slice_from(u"\u0622"):
                                    return False
                            elif among_var == 17:
                                if not self.slice_from(u"\u0624"):
                                    return False
                            elif among_var == 18:
                                if not self.slice_from(u"\u0627"):
                                    return False
                            elif among_var == 19:
                                if not self.slice_from(u"\u0628"):
                                    return False
                            elif among_var == 20:
                                if not self.slice_from(u"\u0629"):
                                    return False
                            elif among_var == 21:
                                if not self.slice_from(u"\u062A"):
                                    return False
                            elif among_var == 22:
                                if not self.slice_from(u"\u062B"):
                                    return False
                            elif among_var == 23:
                                if not self.slice_from(u"\u062C"):
                                    return False
                            elif among_var == 24:
                                if not self.slice_from(u"\u062D"):
                                    return False
                            elif among_var == 25:
                                if not self.slice_from(u"\u062E"):
                                    return False
                            elif among_var == 26:
                                if not self.slice_from(u"\u062F"):
                                    return False
                            elif among_var == 27:
                                if not self.slice_from(u"\u0630"):
                                    return False
                            elif among_var == 28:
                                if not self.slice_from(u"\u0631"):
                                    return False
                            elif among_var == 29:
                                if not self.slice_from(u"\u0632"):
                                    return False
                            elif among_var == 30:
                                if not self.slice_from(u"\u0633"):
                                    return False
                            elif among_var == 31:
                                if not self.slice_from(u"\u0634"):
                                    return False
                            elif among_var == 32:
                                if not self.slice_from(u"\u0635"):
                                    return False
                            elif among_var == 33:
                                if not self.slice_from(u"\u0636"):
                                    return False
                            elif among_var == 34:
                                if not self.slice_from(u"\u0637"):
                                    return False
                            elif among_var == 35:
                                if not self.slice_from(u"\u0638"):
                                    return False
                            elif among_var == 36:
                                if not self.slice_from(u"\u0639"):
                                    return False
                            elif among_var == 37:
                                if not self.slice_from(u"\u063A"):
                                    return False
                            elif among_var == 38:
                                if not self.slice_from(u"\u0641"):
                                    return False
                            elif among_var == 39:
                                if not self.slice_from(u"\u0642"):
                                    return False
                            elif among_var == 40:
                                if not self.slice_from(u"\u0643"):
                                    return False
                            elif among_var == 41:
                                if not self.slice_from(u"\u0644"):
                                    return False
                            elif among_var == 42:
                                if not self.slice_from(u"\u0645"):
                                    return False
                            elif among_var == 43:
                                if not self.slice_from(u"\u0646"):
                                    return False
                            elif among_var == 44:
                                if not self.slice_from(u"\u0647"):
                                    return False
                            elif among_var == 45:
                                if not self.slice_from(u"\u0648"):
                                    return False
                            elif among_var == 46:
                                if not self.slice_from(u"\u0649"):
                                    return False
                            elif among_var == 47:
                                if not self.slice_from(u"\u064A"):
                                    return False
                            elif among_var == 48:
                                if not self.slice_from(u"\u0644\u0627"):
                                    return False
                            elif among_var == 49:
                                if not self.slice_from(u"\u0644\u0623"):
                                    return False
                            elif among_var == 50:
                                if not self.slice_from(u"\u0644\u0625"):
                                    return False
                            else:
                                if not self.slice_from(u"\u0644\u0622"):
                                    return False
                            raise lab2()
                        except lab3: pass
                        self.cursor = v_3
                        if self.cursor >= self.limit:
                            raise lab1()
                        self.cursor += 1
                    except lab2: pass
                    continue
                except lab1: pass
                self.cursor = v_2
                break
        except lab0: pass
        self.cursor = v_1
        return True

    def __r_Normalize_post(self):
        v_1 = self.cursor
        try:
            self.limit_backward = self.cursor
            self.cursor = self.limit
            self.ket = self.cursor
            if self.find_among_b(ArabicStemmer.a_1) == 0:
                raise lab0()
            self.bra = self.cursor
            if not self.slice_from(u"\u0621"):
                return False
            self.cursor = self.limit_backward
        except lab0: pass
        self.cursor = v_1
        v_2 = self.cursor
        try:
            while True:
                v_3 = self.cursor
                try:
                    try:
                        v_4 = self.cursor
                        try:
                            self.bra = self.cursor
                            among_var = self.find_among(ArabicStemmer.a_2)
                            if among_var == 0:
                                raise lab4()
                            self.ket = self.cursor
                            if among_var == 1:
                                if not self.slice_from(u"\u0627"):
                                    return False
                            elif among_var == 2:
                                if not self.slice_from(u"\u0648"):
                                    return False
                            else:
                                if not self.slice_from(u"\u064A"):
                                    return False
                            raise lab3()
                        except lab4: pass
                        self.cursor = v_4
                        if self.cursor >= self.limit:
                            raise lab2()
                        self.cursor += 1
                    except lab3: pass
                    continue
                except lab2: pass
                self.cursor = v_3
                break
        except lab1: pass
        self.cursor = v_2
        return True

    def __r_Checks1(self):
        self.bra = self.cursor
        among_var = self.find_among(ArabicStemmer.a_3)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if among_var == 1:
            if len(self.current) <= 4:
                return False
            self.B_is_noun = True
            self.B_is_verb = False
            self.B_is_defined = True
        else:
            if len(self.current) <= 3:
                return False
            self.B_is_noun = True
            self.B_is_verb = False
            self.B_is_defined = True
        return True

    def __r_Prefix_Step1(self):
        self.bra = self.cursor
        among_var = self.find_among(ArabicStemmer.a_4)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if among_var == 1:
            if len(self.current) <= 3:
                return False
            if not self.slice_from(u"\u0623"):
                return False
        elif among_var == 2:
            if len(self.current) <= 3:
                return False
            if not self.slice_from(u"\u0622"):
                return False
        elif among_var == 3:
            if len(self.current) <= 3:
                return False
            if not self.slice_from(u"\u0627"):
                return False
        else:
            if len(self.current) <= 3:
                return False
            if not self.slice_from(u"\u0625"):
                return False
        return True

    def __r_Prefix_Step2(self):
        self.bra = self.cursor
        if self.find_among(ArabicStemmer.a_5) == 0:
            return False
        self.ket = self.cursor
        if len(self.current) <= 3:
            return False
        v_1 = self.cursor
        try:
            if not self.eq_s(u"\u0627"):
                raise lab0()
            return False
        except lab0: pass
        self.cursor = v_1
        if not self.slice_del():
            return False

        return True

    def __r_Prefix_Step3a_Noun(self):
        self.bra = self.cursor
        among_var = self.find_among(ArabicStemmer.a_6)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if among_var == 1:
            if len(self.current) <= 5:
                return False
            if not self.slice_del():
                return False

        else:
            if len(self.current) <= 4:
                return False
            if not self.slice_del():
                return False

        return True

    def __r_Prefix_Step3b_Noun(self):
        self.bra = self.cursor
        among_var = self.find_among(ArabicStemmer.a_7)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if among_var == 1:
            if len(self.current) <= 3:
                return False
            if not self.slice_del():
                return False

        elif among_var == 2:
            if len(self.current) <= 3:
                return False
            if not self.slice_from(u"\u0628"):
                return False
        elif among_var == 3:
            if len(self.current) <= 3:
                return False
            if not self.slice_from(u"\u0643"):
                return False
        return True

    def __r_Prefix_Step3_Verb(self):
        self.bra = self.cursor
        among_var = self.find_among(ArabicStemmer.a_8)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if among_var == 1:
            if len(self.current) <= 4:
                return False
            if not self.slice_from(u"\u064A"):
                return False
        elif among_var == 2:
            if len(self.current) <= 4:
                return False
            if not self.slice_from(u"\u062A"):
                return False
        elif among_var == 3:
            if len(self.current) <= 4:
                return False
            if not self.slice_from(u"\u0646"):
                return False
        else:
            if len(self.current) <= 4:
                return False
            if not self.slice_from(u"\u0623"):
                return False
        return True

    def __r_Prefix_Step4_Verb(self):
        self.bra = self.cursor
        if self.find_among(ArabicStemmer.a_9) == 0:
            return False
        self.ket = self.cursor
        if len(self.current) <= 4:
            return False
        self.B_is_verb = True
        self.B_is_noun = False
        if not self.slice_from(u"\u0627\u0633\u062A"):
            return False
        return True

    def __r_Suffix_Noun_Step1a(self):
        self.ket = self.cursor
        among_var = self.find_among_b(ArabicStemmer.a_10)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if len(self.current) < 4:
                return False
            if not self.slice_del():
                return False

        elif among_var == 2:
            if len(self.current) < 5:
                return False
            if not self.slice_del():
                return False

        else:
            if len(self.current) < 6:
                return False
            if not self.slice_del():
                return False

        return True

    def __r_Suffix_Noun_Step1b(self):
        self.ket = self.cursor
        if self.find_among_b(ArabicStemmer.a_11) == 0:
            return False
        self.bra = self.cursor
        if len(self.current) <= 5:
            return False
        if not self.slice_del():
            return False

        return True

    def __r_Suffix_Noun_Step2a(self):
        self.ket = self.cursor
        if self.find_among_b(ArabicStemmer.a_12) == 0:
            return False
        self.bra = self.cursor
        if len(self.current) <= 4:
            return False
        if not self.slice_del():
            return False

        return True

    def __r_Suffix_Noun_Step2b(self):
        self.ket = self.cursor
        if self.find_among_b(ArabicStemmer.a_13) == 0:
            return False
        self.bra = self.cursor
        if len(self.current) < 5:
            return False
        if not self.slice_del():
            return False

        return True

    def __r_Suffix_Noun_Step2c1(self):
        self.ket = self.cursor
        if self.find_among_b(ArabicStemmer.a_14) == 0:
            return False
        self.bra = self.cursor
        if len(self.current) < 4:
            return False
        if not self.slice_del():
            return False

        return True

    def __r_Suffix_Noun_Step2c2(self):
        self.ket = self.cursor
        if self.find_among_b(ArabicStemmer.a_15) == 0:
            return False
        self.bra = self.cursor
        if len(self.current) < 4:
            return False
        if not self.slice_del():
            return False

        return True

    def __r_Suffix_Noun_Step3(self):
        self.ket = self.cursor
        if self.find_among_b(ArabicStemmer.a_16) == 0:
            return False
        self.bra = self.cursor
        if len(self.current) < 3:
            return False
        if not self.slice_del():
            return False

        return True

    def __r_Suffix_Verb_Step1(self):
        self.ket = self.cursor
        among_var = self.find_among_b(ArabicStemmer.a_17)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if len(self.current) < 4:
                return False
            if not self.slice_del():
                return False

        elif among_var == 2:
            if len(self.current) < 5:
                return False
            if not self.slice_del():
                return False

        else:
            if len(self.current) < 6:
                return False
            if not self.slice_del():
                return False

        return True

    def __r_Suffix_Verb_Step2a(self):
        self.ket = self.cursor
        among_var = self.find_among_b(ArabicStemmer.a_18)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if len(self.current) < 4:
                return False
            if not self.slice_del():
                return False

        elif among_var == 2:
            if len(self.current) < 5:
                return False
            if not self.slice_del():
                return False

        elif among_var == 3:
            if len(self.current) <= 5:
                return False
            if not self.slice_del():
                return False

        else:
            if len(self.current) < 6:
                return False
            if not self.slice_del():
                return False

        return True

    def __r_Suffix_Verb_Step2b(self):
        self.ket = self.cursor
        if self.find_among_b(ArabicStemmer.a_19) == 0:
            return False
        self.bra = self.cursor
        if len(self.current) < 5:
            return False
        if not self.slice_del():
            return False

        return True

    def __r_Suffix_Verb_Step2c(self):
        self.ket = self.cursor
        among_var = self.find_among_b(ArabicStemmer.a_20)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if len(self.current) < 4:
                return False
            if not self.slice_del():
                return False

        else:
            if len(self.current) < 6:
                return False
            if not self.slice_del():
                return False

        return True

    def __r_Suffix_All_alef_maqsura(self):
        self.ket = self.cursor
        if self.find_among_b(ArabicStemmer.a_21) == 0:
            return False
        self.bra = self.cursor
        if not self.slice_from(u"\u064A"):
            return False
        return True

    def _stem(self):
        self.B_is_noun = True
        self.B_is_verb = True
        self.B_is_defined = False
        v_1 = self.cursor
        self.__r_Checks1()
        self.cursor = v_1
        self.__r_Normalize_pre()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_3 = self.limit - self.cursor
        try:
            try:
                v_4 = self.limit - self.cursor
                try:
                    if not self.B_is_verb:
                        raise lab2()
                    try:
                        v_5 = self.limit - self.cursor
                        try:
                            v_6 = 1
                            while True:
                                v_7 = self.limit - self.cursor
                                try:
                                    if not self.__r_Suffix_Verb_Step1():
                                        raise lab5()
                                    v_6 -= 1
                                    continue
                                except lab5: pass
                                self.cursor = self.limit - v_7
                                break
                            if v_6 > 0:
                                raise lab4()
                            try:
                                v_8 = self.limit - self.cursor
                                try:
                                    if not self.__r_Suffix_Verb_Step2a():
                                        raise lab7()
                                    raise lab6()
                                except lab7: pass
                                self.cursor = self.limit - v_8
                                try:
                                    if not self.__r_Suffix_Verb_Step2c():
                                        raise lab8()
                                    raise lab6()
                                except lab8: pass
                                self.cursor = self.limit - v_8
                                if self.cursor <= self.limit_backward:
                                    raise lab4()
                                self.cursor -= 1
                            except lab6: pass
                            raise lab3()
                        except lab4: pass
                        self.cursor = self.limit - v_5
                        try:
                            if not self.__r_Suffix_Verb_Step2b():
                                raise lab9()
                            raise lab3()
                        except lab9: pass
                        self.cursor = self.limit - v_5
                        if not self.__r_Suffix_Verb_Step2a():
                            raise lab2()
                    except lab3: pass
                    raise lab1()
                except lab2: pass
                self.cursor = self.limit - v_4
                try:
                    if not self.B_is_noun:
                        raise lab10()
                    v_9 = self.limit - self.cursor
                    try:
                        try:
                            v_10 = self.limit - self.cursor
                            try:
                                if not self.__r_Suffix_Noun_Step2c2():
                                    raise lab13()
                                raise lab12()
                            except lab13: pass
                            self.cursor = self.limit - v_10
                            try:
                                try:
                                    if not self.B_is_defined:
                                        raise lab15()
                                    raise lab14()
                                except lab15: pass
                                if not self.__r_Suffix_Noun_Step1a():
                                    raise lab14()
                                try:
                                    v_12 = self.limit - self.cursor
                                    try:
                                        if not self.__r_Suffix_Noun_Step2a():
                                            raise lab17()
                                        raise lab16()
                                    except lab17: pass
                                    self.cursor = self.limit - v_12
                                    try:
                                        if not self.__r_Suffix_Noun_Step2b():
                                            raise lab18()
                                        raise lab16()
                                    except lab18: pass
                                    self.cursor = self.limit - v_12
                                    try:
                                        if not self.__r_Suffix_Noun_Step2c1():
                                            raise lab19()
                                        raise lab16()
                                    except lab19: pass
                                    self.cursor = self.limit - v_12
                                    if self.cursor <= self.limit_backward:
                                        raise lab14()
                                    self.cursor -= 1
                                except lab16: pass
                                raise lab12()
                            except lab14: pass
                            self.cursor = self.limit - v_10
                            try:
                                if not self.__r_Suffix_Noun_Step1b():
                                    raise lab20()
                                try:
                                    v_13 = self.limit - self.cursor
                                    try:
                                        if not self.__r_Suffix_Noun_Step2a():
                                            raise lab22()
                                        raise lab21()
                                    except lab22: pass
                                    self.cursor = self.limit - v_13
                                    try:
                                        if not self.__r_Suffix_Noun_Step2b():
                                            raise lab23()
                                        raise lab21()
                                    except lab23: pass
                                    self.cursor = self.limit - v_13
                                    if not self.__r_Suffix_Noun_Step2c1():
                                        raise lab20()
                                except lab21: pass
                                raise lab12()
                            except lab20: pass
                            self.cursor = self.limit - v_10
                            try:
                                try:
                                    if not self.B_is_defined:
                                        raise lab25()
                                    raise lab24()
                                except lab25: pass
                                if not self.__r_Suffix_Noun_Step2a():
                                    raise lab24()
                                raise lab12()
                            except lab24: pass
                            self.cursor = self.limit - v_10
                            if not self.__r_Suffix_Noun_Step2b():
                                self.cursor = self.limit - v_9
                                raise lab11()
                        except lab12: pass
                    except lab11: pass
                    if not self.__r_Suffix_Noun_Step3():
                        raise lab10()
                    raise lab1()
                except lab10: pass
                self.cursor = self.limit - v_4
                if not self.__r_Suffix_All_alef_maqsura():
                    raise lab0()
            except lab1: pass
        except lab0: pass
        self.cursor = self.limit - v_3
        self.cursor = self.limit_backward
        v_15 = self.cursor
        try:
            v_16 = self.cursor
            try:
                if not self.__r_Prefix_Step1():
                    self.cursor = v_16
                    raise lab27()
            except lab27: pass
            v_17 = self.cursor
            try:
                if not self.__r_Prefix_Step2():
                    self.cursor = v_17
                    raise lab28()
            except lab28: pass
            try:
                v_18 = self.cursor
                try:
                    if not self.__r_Prefix_Step3a_Noun():
                        raise lab30()
                    raise lab29()
                except lab30: pass
                self.cursor = v_18
                try:
                    if not self.B_is_noun:
                        raise lab31()
                    if not self.__r_Prefix_Step3b_Noun():
                        raise lab31()
                    raise lab29()
                except lab31: pass
                self.cursor = v_18
                if not self.B_is_verb:
                    raise lab26()
                v_19 = self.cursor
                try:
                    if not self.__r_Prefix_Step3_Verb():
                        self.cursor = v_19
                        raise lab32()
                except lab32: pass
                if not self.__r_Prefix_Step4_Verb():
                    raise lab26()
            except lab29: pass
        except lab26: pass
        self.cursor = v_15
        self.__r_Normalize_post()
        return True

    a_0 = [
        Among(u"\u0640", -1, 1),
        Among(u"\u064B", -1, 1),
        Among(u"\u064C", -1, 1),
        Among(u"\u064D", -1, 1),
        Among(u"\u064E", -1, 1),
        Among(u"\u064F", -1, 1),
        Among(u"\u0650", -1, 1),
        Among(u"\u0651", -1, 1),
        Among(u"\u0652", -1, 1),
        Among(u"\u0660", -1, 2),
        Among(u"\u0661", -1, 3),
        Among(u"\u0662", -1, 4),
        Among(u"\u0663", -1, 5),
        Among(u"\u0664", -1, 6),
        Among(u"\u0665", -1, 7),
        Among(u"\u0666", -1, 8),
        Among(u"\u0667", -1, 9),
        Among(u"\u0668", -1, 10),
        Among(u"\u0669", -1, 11),
        Among(u"\uFE80", -1, 12),
        Among(u"\uFE81", -1, 16),
        Among(u"\uFE82", -1, 16),
        Among(u"\uFE83", -1, 13),
        Among(u"\uFE84", -1, 13),
        Among(u"\uFE85", -1, 17),
        Among(u"\uFE86", -1, 17),
        Among(u"\uFE87", -1, 14),
        Among(u"\uFE88", -1, 14),
        Among(u"\uFE89", -1, 15),
        Among(u"\uFE8A", -1, 15),
        Among(u"\uFE8B", -1, 15),
        Among(u"\uFE8C", -1, 15),
        Among(u"\uFE8D", -1, 18),
        Among(u"\uFE8E", -1, 18),
        Among(u"\uFE8F", -1, 19),
        Among(u"\uFE90", -1, 19),
        Among(u"\uFE91", -1, 19),
        Among(u"\uFE92", -1, 19),
        Among(u"\uFE93", -1, 20),
        Among(u"\uFE94", -1, 20),
        Among(u"\uFE95", -1, 21),
        Among(u"\uFE96", -1, 21),
        Among(u"\uFE97", -1, 21),
        Among(u"\uFE98", -1, 21),
        Among(u"\uFE99", -1, 22),
        Among(u"\uFE9A", -1, 22),
        Among(u"\uFE9B", -1, 22),
        Among(u"\uFE9C", -1, 22),
        Among(u"\uFE9D", -1, 23),
        Among(u"\uFE9E", -1, 23),
        Among(u"\uFE9F", -1, 23),
        Among(u"\uFEA0", -1, 23),
        Among(u"\uFEA1", -1, 24),
        Among(u"\uFEA2", -1, 24),
        Among(u"\uFEA3", -1, 24),
        Among(u"\uFEA4", -1, 24),
        Among(u"\uFEA5", -1, 25),
        Among(u"\uFEA6", -1, 25),
        Among(u"\uFEA7", -1, 25),
        Among(u"\uFEA8", -1, 25),
        Among(u"\uFEA9", -1, 26),
        Among(u"\uFEAA", -1, 26),
        Among(u"\uFEAB", -1, 27),
        Among(u"\uFEAC", -1, 27),
        Among(u"\uFEAD", -1, 28),
        Among(u"\uFEAE", -1, 28),
        Among(u"\uFEAF", -1, 29),
        Among(u"\uFEB0", -1, 29),
        Among(u"\uFEB1", -1, 30),
        Among(u"\uFEB2", -1, 30),
        Among(u"\uFEB3", -1, 30),
        Among(u"\uFEB4", -1, 30),
        Among(u"\uFEB5", -1, 31),
        Among(u"\uFEB6", -1, 31),
        Among(u"\uFEB7", -1, 31),
        Among(u"\uFEB8", -1, 31),
        Among(u"\uFEB9", -1, 32),
        Among(u"\uFEBA", -1, 32),
        Among(u"\uFEBB", -1, 32),
        Among(u"\uFEBC", -1, 32),
        Among(u"\uFEBD", -1, 33),
        Among(u"\uFEBE", -1, 33),
        Among(u"\uFEBF", -1, 33),
        Among(u"\uFEC0", -1, 33),
        Among(u"\uFEC1", -1, 34),
        Among(u"\uFEC2", -1, 34),
        Among(u"\uFEC3", -1, 34),
        Among(u"\uFEC4", -1, 34),
        Among(u"\uFEC5", -1, 35),
        Among(u"\uFEC6", -1, 35),
        Among(u"\uFEC7", -1, 35),
        Among(u"\uFEC8", -1, 35),
        Among(u"\uFEC9", -1, 36),
        Among(u"\uFECA", -1, 36),
        Among(u"\uFECB", -1, 36),
        Among(u"\uFECC", -1, 36),
        Among(u"\uFECD", -1, 37),
        Among(u"\uFECE", -1, 37),
        Among(u"\uFECF", -1, 37),
        Among(u"\uFED0", -1, 37),
        Among(u"\uFED1", -1, 38),
        Among(u"\uFED2", -1, 38),
        Among(u"\uFED3", -1, 38),
        Among(u"\uFED4", -1, 38),
        Among(u"\uFED5", -1, 39),
        Among(u"\uFED6", -1, 39),
        Among(u"\uFED7", -1, 39),
        Among(u"\uFED8", -1, 39),
        Among(u"\uFED9", -1, 40),
        Among(u"\uFEDA", -1, 40),
        Among(u"\uFEDB", -1, 40),
        Among(u"\uFEDC", -1, 40),
        Among(u"\uFEDD", -1, 41),
        Among(u"\uFEDE", -1, 41),
        Among(u"\uFEDF", -1, 41),
        Among(u"\uFEE0", -1, 41),
        Among(u"\uFEE1", -1, 42),
        Among(u"\uFEE2", -1, 42),
        Among(u"\uFEE3", -1, 42),
        Among(u"\uFEE4", -1, 42),
        Among(u"\uFEE5", -1, 43),
        Among(u"\uFEE6", -1, 43),
        Among(u"\uFEE7", -1, 43),
        Among(u"\uFEE8", -1, 43),
        Among(u"\uFEE9", -1, 44),
        Among(u"\uFEEA", -1, 44),
        Among(u"\uFEEB", -1, 44),
        Among(u"\uFEEC", -1, 44),
        Among(u"\uFEED", -1, 45),
        Among(u"\uFEEE", -1, 45),
        Among(u"\uFEEF", -1, 46),
        Among(u"\uFEF0", -1, 46),
        Among(u"\uFEF1", -1, 47),
        Among(u"\uFEF2", -1, 47),
        Among(u"\uFEF3", -1, 47),
        Among(u"\uFEF4", -1, 47),
        Among(u"\uFEF5", -1, 51),
        Among(u"\uFEF6", -1, 51),
        Among(u"\uFEF7", -1, 49),
        Among(u"\uFEF8", -1, 49),
        Among(u"\uFEF9", -1, 50),
        Among(u"\uFEFA", -1, 50),
        Among(u"\uFEFB", -1, 48),
        Among(u"\uFEFC", -1, 48)
    ]

    a_1 = [
        Among(u"\u0622", -1, 1),
        Among(u"\u0623", -1, 1),
        Among(u"\u0624", -1, 1),
        Among(u"\u0625", -1, 1),
        Among(u"\u0626", -1, 1)
    ]

    a_2 = [
        Among(u"\u0622", -1, 1),
        Among(u"\u0623", -1, 1),
        Among(u"\u0624", -1, 2),
        Among(u"\u0625", -1, 1),
        Among(u"\u0626", -1, 3)
    ]

    a_3 = [
        Among(u"\u0627\u0644", -1, 2),
        Among(u"\u0628\u0627\u0644", -1, 1),
        Among(u"\u0643\u0627\u0644", -1, 1),
        Among(u"\u0644\u0644", -1, 2)
    ]

    a_4 = [
        Among(u"\u0623\u0622", -1, 2),
        Among(u"\u0623\u0623", -1, 1),
        Among(u"\u0623\u0624", -1, 1),
        Among(u"\u0623\u0625", -1, 4),
        Among(u"\u0623\u0627", -1, 3)
    ]

    a_5 = [
        Among(u"\u0641", -1, 1),
        Among(u"\u0648", -1, 1)
    ]

    a_6 = [
        Among(u"\u0627\u0644", -1, 2),
        Among(u"\u0628\u0627\u0644", -1, 1),
        Among(u"\u0643\u0627\u0644", -1, 1),
        Among(u"\u0644\u0644", -1, 2)
    ]

    a_7 = [
        Among(u"\u0628", -1, 1),
        Among(u"\u0628\u0627", 0, -1),
        Among(u"\u0628\u0628", 0, 2),
        Among(u"\u0643\u0643", -1, 3)
    ]

    a_8 = [
        Among(u"\u0633\u0623", -1, 4),
        Among(u"\u0633\u062A", -1, 2),
        Among(u"\u0633\u0646", -1, 3),
        Among(u"\u0633\u064A", -1, 1)
    ]

    a_9 = [
        Among(u"\u062A\u0633\u062A", -1, 1),
        Among(u"\u0646\u0633\u062A", -1, 1),
        Among(u"\u064A\u0633\u062A", -1, 1)
    ]

    a_10 = [
        Among(u"\u0643\u0645\u0627", -1, 3),
        Among(u"\u0647\u0645\u0627", -1, 3),
        Among(u"\u0646\u0627", -1, 2),
        Among(u"\u0647\u0627", -1, 2),
        Among(u"\u0643", -1, 1),
        Among(u"\u0643\u0645", -1, 2),
        Among(u"\u0647\u0645", -1, 2),
        Among(u"\u0647\u0646", -1, 2),
        Among(u"\u0647", -1, 1),
        Among(u"\u064A", -1, 1)
    ]

    a_11 = [
        Among(u"\u0646", -1, 1)
    ]

    a_12 = [
        Among(u"\u0627", -1, 1),
        Among(u"\u0648", -1, 1),
        Among(u"\u064A", -1, 1)
    ]

    a_13 = [
        Among(u"\u0627\u062A", -1, 1)
    ]

    a_14 = [
        Among(u"\u062A", -1, 1)
    ]

    a_15 = [
        Among(u"\u0629", -1, 1)
    ]

    a_16 = [
        Among(u"\u064A", -1, 1)
    ]

    a_17 = [
        Among(u"\u0643\u0645\u0627", -1, 3),
        Among(u"\u0647\u0645\u0627", -1, 3),
        Among(u"\u0646\u0627", -1, 2),
        Among(u"\u0647\u0627", -1, 2),
        Among(u"\u0643", -1, 1),
        Among(u"\u0643\u0645", -1, 2),
        Among(u"\u0647\u0645", -1, 2),
        Among(u"\u0643\u0646", -1, 2),
        Among(u"\u0647\u0646", -1, 2),
        Among(u"\u0647", -1, 1),
        Among(u"\u0643\u0645\u0648", -1, 3),
        Among(u"\u0646\u064A", -1, 2)
    ]

    a_18 = [
        Among(u"\u0627", -1, 1),
        Among(u"\u062A\u0627", 0, 2),
        Among(u"\u062A\u0645\u0627", 0, 4),
        Among(u"\u0646\u0627", 0, 2),
        Among(u"\u062A", -1, 1),
        Among(u"\u0646", -1, 1),
        Among(u"\u0627\u0646", 5, 3),
        Among(u"\u062A\u0646", 5, 2),
        Among(u"\u0648\u0646", 5, 3),
        Among(u"\u064A\u0646", 5, 3),
        Among(u"\u064A", -1, 1)
    ]

    a_19 = [
        Among(u"\u0648\u0627", -1, 1),
        Among(u"\u062A\u0645", -1, 1)
    ]

    a_20 = [
        Among(u"\u0648", -1, 1),
        Among(u"\u062A\u0645\u0648", 0, 2)
    ]

    a_21 = [
        Among(u"\u0649", -1, 1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass


class lab4(BaseException): pass


class lab5(BaseException): pass


class lab6(BaseException): pass


class lab7(BaseException): pass


class lab8(BaseException): pass


class lab9(BaseException): pass


class lab10(BaseException): pass


class lab11(BaseException): pass


class lab12(BaseException): pass


class lab13(BaseException): pass


class lab14(BaseException): pass


class lab15(BaseException): pass


class lab16(BaseException): pass


class lab17(BaseException): pass


class lab18(BaseException): pass


class lab19(BaseException): pass


class lab20(BaseException): pass


class lab21(BaseException): pass


class lab22(BaseException): pass


class lab23(BaseException): pass


class lab24(BaseException): pass


class lab25(BaseException): pass


class lab26(BaseException): pass


class lab27(BaseException): pass


class lab28(BaseException): pass


class lab29(BaseException): pass


class lab30(BaseException): pass


class lab31(BaseException): pass


class lab32(BaseException): pass



#-*- coding: utf-8 -*-
# Generated from esperanto.sbl by Snowball 3.0.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class EsperantoStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from esperanto.sbl by Snowball 3.0.1 - https://snowballstem.org/
    '''

    g_vowel = {u"a", u"e", u"i", u"o", u"u"}

    g_aou = {u"a", u"o", u"u"}

    g_digit = {u"0", u"1", u"2", u"3", u"4", u"5", u"6", u"7", u"8", u"9"}

    B_foreign = False

    def __r_canonical_form(self):
        self.B_foreign = False
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(EsperantoStemmer.a_0)
                self.ket = self.cursor
                if among_var == 1:
                    if not self.slice_from(u"ĉ"):
                        return False
                elif among_var == 2:
                    if not self.slice_from(u"ĝ"):
                        return False
                elif among_var == 3:
                    if not self.slice_from(u"ĥ"):
                        return False
                elif among_var == 4:
                    if not self.slice_from(u"ĵ"):
                        return False
                elif among_var == 5:
                    if not self.slice_from(u"ŝ"):
                        return False
                elif among_var == 6:
                    if not self.slice_from(u"ŭ"):
                        return False
                elif among_var == 7:
                    if not self.slice_from(u"a"):
                        return False
                    self.B_foreign = True
                elif among_var == 8:
                    if not self.slice_from(u"e"):
                        return False
                    self.B_foreign = True
                elif among_var == 9:
                    if not self.slice_from(u"i"):
                        return False
                    self.B_foreign = True
                elif among_var == 10:
                    if not self.slice_from(u"o"):
                        return False
                    self.B_foreign = True
                elif among_var == 11:
                    if not self.slice_from(u"u"):
                        return False
                    self.B_foreign = True
                elif among_var == 12:
                    self.B_foreign = True
                elif among_var == 13:
                    self.B_foreign = False
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        try:
            if not self.B_foreign:
                raise lab1()
            return False
        except lab1: pass
        return True

    def __r_initial_apostrophe(self):
        self.bra = self.cursor
        if not self.eq_s(u"'"):
            return False
        self.ket = self.cursor
        if not self.eq_s(u"st"):
            return False
        if self.find_among(EsperantoStemmer.a_1) == 0:
            return False
        if self.cursor < self.limit:
            return False
        if not self.slice_from(u"e"):
            return False
        return True

    def __r_pronoun(self):
        self.ket = self.cursor
        v_1 = self.limit - self.cursor
        try:
            if not self.eq_s_b(u"n"):
                self.cursor = self.limit - v_1
                raise lab0()
        except lab0: pass
        self.bra = self.cursor
        if self.find_among_b(EsperantoStemmer.a_2) == 0:
            return False
        try:
            v_2 = self.limit - self.cursor
            try:
                if self.cursor > self.limit_backward:
                    raise lab2()
                raise lab1()
            except lab2: pass
            self.cursor = self.limit - v_2
            if not self.eq_s_b(u"-"):
                return False
        except lab1: pass
        if not self.slice_del():
            return False

        return True

    def __r_final_apostrophe(self):
        self.ket = self.cursor
        if not self.eq_s_b(u"'"):
            return False
        self.bra = self.cursor
        try:
            v_1 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"l"):
                    raise lab1()
                if self.cursor > self.limit_backward:
                    raise lab1()
                if not self.slice_from(u"a"):
                    return False
                raise lab0()
            except lab1: pass
            self.cursor = self.limit - v_1
            try:
                if not self.eq_s_b(u"un"):
                    raise lab2()
                if self.cursor > self.limit_backward:
                    raise lab2()
                if not self.slice_from(u"u"):
                    return False
                raise lab0()
            except lab2: pass
            self.cursor = self.limit - v_1
            try:
                if self.find_among_b(EsperantoStemmer.a_3) == 0:
                    raise lab3()
                try:
                    v_2 = self.limit - self.cursor
                    try:
                        if self.cursor > self.limit_backward:
                            raise lab5()
                        raise lab4()
                    except lab5: pass
                    self.cursor = self.limit - v_2
                    if not self.eq_s_b(u"-"):
                        raise lab3()
                except lab4: pass
                if not self.slice_from(u"aŭ"):
                    return False
                raise lab0()
            except lab3: pass
            self.cursor = self.limit - v_1
            if not self.slice_from(u"o"):
                return False
        except lab0: pass
        return True

    def __r_ujn_suffix(self):
        self.ket = self.cursor
        v_1 = self.limit - self.cursor
        try:
            if not self.eq_s_b(u"n"):
                self.cursor = self.limit - v_1
                raise lab0()
        except lab0: pass
        v_2 = self.limit - self.cursor
        try:
            if not self.eq_s_b(u"j"):
                self.cursor = self.limit - v_2
                raise lab1()
        except lab1: pass
        self.bra = self.cursor
        if self.find_among_b(EsperantoStemmer.a_4) == 0:
            return False
        try:
            v_3 = self.limit - self.cursor
            try:
                if self.cursor > self.limit_backward:
                    raise lab3()
                raise lab2()
            except lab3: pass
            self.cursor = self.limit - v_3
            if not self.eq_s_b(u"-"):
                return False
        except lab2: pass
        if not self.slice_del():
            return False

        return True

    def __r_uninflected(self):
        if self.find_among_b(EsperantoStemmer.a_5) == 0:
            return False
        try:
            v_1 = self.limit - self.cursor
            try:
                if self.cursor > self.limit_backward:
                    raise lab1()
                raise lab0()
            except lab1: pass
            self.cursor = self.limit - v_1
            if not self.eq_s_b(u"-"):
                return False
        except lab0: pass
        return True

    def __r_merged_numeral(self):
        if self.find_among_b(EsperantoStemmer.a_6) == 0:
            return False
        if self.find_among_b(EsperantoStemmer.a_7) == 0:
            return False
        return True

    def __r_correlative(self):
        self.ket = self.cursor
        self.bra = self.cursor
        v_1 = self.limit - self.cursor
        try:
            v_2 = self.limit - self.cursor
            try:
                v_3 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"n"):
                        self.cursor = self.limit - v_3
                        raise lab2()
                except lab2: pass
                self.bra = self.cursor
                if not self.eq_s_b(u"e"):
                    raise lab1()
                raise lab0()
            except lab1: pass
            self.cursor = self.limit - v_2
            v_4 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"n"):
                    self.cursor = self.limit - v_4
                    raise lab3()
            except lab3: pass
            v_5 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"j"):
                    self.cursor = self.limit - v_5
                    raise lab4()
            except lab4: pass
            self.bra = self.cursor
            if not self.in_grouping_b(EsperantoStemmer.g_aou):
                return False
        except lab0: pass
        if not self.eq_s_b(u"i"):
            return False
        v_6 = self.limit - self.cursor
        try:
            if self.find_among_b(EsperantoStemmer.a_8) == 0:
                self.cursor = self.limit - v_6
                raise lab5()
        except lab5: pass
        try:
            v_7 = self.limit - self.cursor
            try:
                if self.cursor > self.limit_backward:
                    raise lab7()
                raise lab6()
            except lab7: pass
            self.cursor = self.limit - v_7
            if not self.eq_s_b(u"-"):
                return False
        except lab6: pass
        self.cursor = self.limit - v_1
        if not self.slice_del():
            return False

        return True

    def __r_long_word(self):
        try:
            v_1 = self.limit - self.cursor
            try:
                for v_2 in 0, 0:

                    if not self.go_out_grouping_b(EsperantoStemmer.g_vowel):
                        raise lab1()
                    self.cursor -= 1
                raise lab0()
            except lab1: pass
            self.cursor = self.limit - v_1
            try:
                try:
                    while True:
                        try:
                            if not self.eq_s_b(u"-"):
                                raise lab4()
                            raise lab3()
                        except lab4: pass
                        if self.cursor <= self.limit_backward:
                            raise lab2()
                        self.cursor -= 1
                except lab3: pass
                if self.cursor <= self.limit_backward:
                    raise lab2()
                self.cursor -= 1
                raise lab0()
            except lab2: pass
            self.cursor = self.limit - v_1
            if not self.go_out_grouping_b(EsperantoStemmer.g_digit):
                return False
            self.cursor -= 1
        except lab0: pass
        return True

    def __r_not_after_letter(self):
        try:
            v_1 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"-"):
                    raise lab1()
                raise lab0()
            except lab1: pass
            self.cursor = self.limit - v_1
            if not self.in_grouping_b(EsperantoStemmer.g_digit):
                return False
        except lab0: pass
        return True

    def __r_standard_suffix(self):
        self.ket = self.cursor
        if self.find_among_b(EsperantoStemmer.a_9) == 0:
            return False
        v_1 = self.limit - self.cursor
        try:
            if not self.eq_s_b(u"-"):
                self.cursor = self.limit - v_1
                raise lab0()
        except lab0: pass
        self.bra = self.cursor
        if not self.slice_del():
            return False

        return True

    def _stem(self):
        v_1 = self.cursor
        if not self.__r_canonical_form():
            return False
        self.cursor = v_1
        v_2 = self.cursor
        self.__r_initial_apostrophe()
        self.cursor = v_2
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_3 = self.limit - self.cursor
        try:
            if not self.__r_pronoun():
                raise lab0()
            return False
        except lab0: pass
        self.cursor = self.limit - v_3
        v_4 = self.limit - self.cursor
        self.__r_final_apostrophe()
        self.cursor = self.limit - v_4
        v_5 = self.limit - self.cursor
        try:
            if not self.__r_correlative():
                raise lab1()
            return False
        except lab1: pass
        self.cursor = self.limit - v_5
        v_6 = self.limit - self.cursor
        try:
            if not self.__r_uninflected():
                raise lab2()
            return False
        except lab2: pass
        self.cursor = self.limit - v_6
        v_7 = self.limit - self.cursor
        try:
            if not self.__r_merged_numeral():
                raise lab3()
            return False
        except lab3: pass
        self.cursor = self.limit - v_7
        v_8 = self.limit - self.cursor
        try:
            if not self.__r_ujn_suffix():
                raise lab4()
            return False
        except lab4: pass
        self.cursor = self.limit - v_8
        v_9 = self.limit - self.cursor
        if not self.__r_long_word():
            return False
        self.cursor = self.limit - v_9
        if not self.__r_standard_suffix():
            return False
        self.cursor = self.limit_backward
        return True

    a_0 = [
        Among(u"", -1, 14),
        Among(u"-", 0, 13),
        Among(u"cx", 0, 1),
        Among(u"gx", 0, 2),
        Among(u"hx", 0, 3),
        Among(u"jx", 0, 4),
        Among(u"q", 0, 12),
        Among(u"sx", 0, 5),
        Among(u"ux", 0, 6),
        Among(u"w", 0, 12),
        Among(u"x", 0, 12),
        Among(u"y", 0, 12),
        Among(u"á", 0, 7),
        Among(u"é", 0, 8),
        Among(u"í", 0, 9),
        Among(u"ó", 0, 10),
        Among(u"ú", 0, 11)
    ]

    a_1 = [
        Among(u"as", -1, -1),
        Among(u"i", -1, -1),
        Among(u"is", 1, -1),
        Among(u"os", -1, -1),
        Among(u"u", -1, -1),
        Among(u"us", 4, -1)
    ]

    a_2 = [
        Among(u"ci", -1, -1),
        Among(u"gi", -1, -1),
        Among(u"hi", -1, -1),
        Among(u"li", -1, -1),
        Among(u"ili", 3, -1),
        Among(u"ŝli", 3, -1),
        Among(u"mi", -1, -1),
        Among(u"ni", -1, -1),
        Among(u"oni", 7, -1),
        Among(u"ri", -1, -1),
        Among(u"si", -1, -1),
        Among(u"vi", -1, -1),
        Among(u"ivi", 11, -1),
        Among(u"ĝi", -1, -1),
        Among(u"ŝi", -1, -1),
        Among(u"iŝi", 14, -1),
        Among(u"malŝi", 14, -1)
    ]

    a_3 = [
        Among(u"amb", -1, -1),
        Among(u"bald", -1, -1),
        Among(u"malbald", 1, -1),
        Among(u"morg", -1, -1),
        Among(u"postmorg", 3, -1),
        Among(u"adi", -1, -1),
        Among(u"hodi", -1, -1),
        Among(u"ank", -1, -1),
        Among(u"ĉirk", -1, -1),
        Among(u"tutĉirk", 8, -1),
        Among(u"presk", -1, -1),
        Among(u"almen", -1, -1),
        Among(u"apen", -1, -1),
        Among(u"hier", -1, -1),
        Among(u"antaŭhier", 13, -1),
        Among(u"malgr", -1, -1),
        Among(u"ankor", -1, -1),
        Among(u"kontr", -1, -1),
        Among(u"anstat", -1, -1),
        Among(u"kvaz", -1, -1)
    ]

    a_4 = [
        Among(u"aliu", -1, -1),
        Among(u"unu", -1, -1)
    ]

    a_5 = [
        Among(u"aha", -1, -1),
        Among(u"haha", 0, -1),
        Among(u"haleluja", -1, -1),
        Among(u"hola", -1, -1),
        Among(u"hosana", -1, -1),
        Among(u"maltra", -1, -1),
        Among(u"hura", -1, -1),
        Among(u"ĥaĥa", -1, -1),
        Among(u"ekde", -1, -1),
        Among(u"elde", -1, -1),
        Among(u"disde", -1, -1),
        Among(u"ehe", -1, -1),
        Among(u"maltre", -1, -1),
        Among(u"dirlididi", -1, -1),
        Among(u"malpli", -1, -1),
        Among(u"malĉi", -1, -1),
        Among(u"malkaj", -1, -1),
        Among(u"amen", -1, -1),
        Among(u"tamen", 17, -1),
        Among(u"oho", -1, -1),
        Among(u"maltro", -1, -1),
        Among(u"minus", -1, -1),
        Among(u"uhu", -1, -1),
        Among(u"muu", -1, -1)
    ]

    a_6 = [
        Among(u"tri", -1, -1),
        Among(u"du", -1, -1),
        Among(u"unu", -1, -1)
    ]

    a_7 = [
        Among(u"dek", -1, -1),
        Among(u"cent", -1, -1)
    ]

    a_8 = [
        Among(u"k", -1, -1),
        Among(u"kelk", 0, -1),
        Among(u"nen", -1, -1),
        Among(u"t", -1, -1),
        Among(u"mult", 3, -1),
        Among(u"samt", 3, -1),
        Among(u"ĉ", -1, -1)
    ]

    a_9 = [
        Among(u"a", -1, -1),
        Among(u"e", -1, -1),
        Among(u"i", -1, -1),
        Among(u"j", -1, -1, __r_not_after_letter),
        Among(u"aj", 3, -1),
        Among(u"oj", 3, -1),
        Among(u"n", -1, -1, __r_not_after_letter),
        Among(u"an", 6, -1),
        Among(u"en", 6, -1),
        Among(u"jn", 6, -1, __r_not_after_letter),
        Among(u"ajn", 9, -1),
        Among(u"ojn", 9, -1),
        Among(u"on", 6, -1),
        Among(u"o", -1, -1),
        Among(u"as", -1, -1),
        Among(u"is", -1, -1),
        Among(u"os", -1, -1),
        Among(u"us", -1, -1),
        Among(u"u", -1, -1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass


class lab4(BaseException): pass


class lab5(BaseException): pass


class lab6(BaseException): pass


class lab7(BaseException): pass



#-*- coding: utf-8 -*-
# Generated from russian.sbl by Snowball 3.0.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class RussianStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from russian.sbl by Snowball 3.0.1 - https://snowballstem.org/
    '''

    g_v = {u"а", u"е", u"и", u"о", u"у", u"ы", u"э", u"ю", u"я"}

    I_p2 = 0
    I_pV = 0

    def __r_mark_regions(self):
        self.I_pV = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            if not self.go_out_grouping(RussianStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_pV = self.cursor
            if not self.go_in_grouping(RussianStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_out_grouping(RussianStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(RussianStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab0: pass
        self.cursor = v_1
        return True

    def __r_R2(self):
        return self.I_p2 <= self.cursor

    def __r_perfective_gerund(self):
        self.ket = self.cursor
        among_var = self.find_among_b(RussianStemmer.a_0)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            try:
                v_1 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"а"):
                        raise lab1()
                    raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_1
                if not self.eq_s_b(u"я"):
                    return False
            except lab0: pass
            if not self.slice_del():
                return False

        else:
            if not self.slice_del():
                return False

        return True

    def __r_adjective(self):
        self.ket = self.cursor
        if self.find_among_b(RussianStemmer.a_1) == 0:
            return False
        self.bra = self.cursor
        if not self.slice_del():
            return False

        return True

    def __r_adjectival(self):
        if not self.__r_adjective():
            return False
        v_1 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(RussianStemmer.a_2)
            if among_var == 0:
                self.cursor = self.limit - v_1
                raise lab0()
            self.bra = self.cursor
            if among_var == 1:
                try:
                    v_2 = self.limit - self.cursor
                    try:
                        if not self.eq_s_b(u"а"):
                            raise lab2()
                        raise lab1()
                    except lab2: pass
                    self.cursor = self.limit - v_2
                    if not self.eq_s_b(u"я"):
                        self.cursor = self.limit - v_1
                        raise lab0()
                except lab1: pass
                if not self.slice_del():
                    return False

            else:
                if not self.slice_del():
                    return False

        except lab0: pass
        return True

    def __r_reflexive(self):
        self.ket = self.cursor
        if self.find_among_b(RussianStemmer.a_3) == 0:
            return False
        self.bra = self.cursor
        if not self.slice_del():
            return False

        return True

    def __r_verb(self):
        self.ket = self.cursor
        among_var = self.find_among_b(RussianStemmer.a_4)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            try:
                v_1 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"а"):
                        raise lab1()
                    raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_1
                if not self.eq_s_b(u"я"):
                    return False
            except lab0: pass
            if not self.slice_del():
                return False

        else:
            if not self.slice_del():
                return False

        return True

    def __r_noun(self):
        self.ket = self.cursor
        if self.find_among_b(RussianStemmer.a_5) == 0:
            return False
        self.bra = self.cursor
        if not self.slice_del():
            return False

        return True

    def __r_derivational(self):
        self.ket = self.cursor
        if self.find_among_b(RussianStemmer.a_6) == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R2():
            return False
        if not self.slice_del():
            return False

        return True

    def __r_tidy_up(self):
        self.ket = self.cursor
        among_var = self.find_among_b(RussianStemmer.a_7)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.slice_del():
                return False

            self.ket = self.cursor
            if not self.eq_s_b(u"н"):
                return False
            self.bra = self.cursor
            if not self.eq_s_b(u"н"):
                return False
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.eq_s_b(u"н"):
                return False
            if not self.slice_del():
                return False

        else:
            if not self.slice_del():
                return False

        return True

    def _stem(self):
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    try:
                        while True:
                            v_3 = self.cursor
                            try:
                                self.bra = self.cursor
                                if not self.eq_s(u"ё"):
                                    raise lab3()
                                self.ket = self.cursor
                                self.cursor = v_3
                                raise lab2()
                            except lab3: pass
                            self.cursor = v_3
                            if self.cursor >= self.limit:
                                raise lab1()
                            self.cursor += 1
                    except lab2: pass
                    if not self.slice_from(u"е"):
                        return False
                    continue
                except lab1: pass
                self.cursor = v_2
                break
        except lab0: pass
        self.cursor = v_1
        self.__r_mark_regions()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        if self.cursor < self.I_pV:
            return False
        v_6 = self.limit_backward
        self.limit_backward = self.I_pV
        v_7 = self.limit - self.cursor
        try:
            try:
                v_8 = self.limit - self.cursor
                try:
                    if not self.__r_perfective_gerund():
                        raise lab6()
                    raise lab5()
                except lab6: pass
                self.cursor = self.limit - v_8
                v_9 = self.limit - self.cursor
                try:
                    if not self.__r_reflexive():
                        self.cursor = self.limit - v_9
                        raise lab7()
                except lab7: pass
                try:
                    v_10 = self.limit - self.cursor
                    try:
                        if not self.__r_adjectival():
                            raise lab9()
                        raise lab8()
                    except lab9: pass
                    self.cursor = self.limit - v_10
                    try:
                        if not self.__r_verb():
                            raise lab10()
                        raise lab8()
                    except lab10: pass
                    self.cursor = self.limit - v_10
                    if not self.__r_noun():
                        raise lab4()
                except lab8: pass
            except lab5: pass
        except lab4: pass
        self.cursor = self.limit - v_7
        v_11 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if not self.eq_s_b(u"и"):
                self.cursor = self.limit - v_11
                raise lab11()
            self.bra = self.cursor
            if not self.slice_del():
                return False

        except lab11: pass
        v_12 = self.limit - self.cursor
        self.__r_derivational()
        self.cursor = self.limit - v_12
        v_13 = self.limit - self.cursor
        self.__r_tidy_up()
        self.cursor = self.limit - v_13
        self.limit_backward = v_6
        self.cursor = self.limit_backward
        return True

    a_0 = [
        Among(u"в", -1, 1),
        Among(u"ив", 0, 2),
        Among(u"ыв", 0, 2),
        Among(u"вши", -1, 1),
        Among(u"ивши", 3, 2),
        Among(u"ывши", 3, 2),
        Among(u"вшись", -1, 1),
        Among(u"ившись", 6, 2),
        Among(u"ывшись", 6, 2)
    ]

    a_1 = [
        Among(u"ее", -1, 1),
        Among(u"ие", -1, 1),
        Among(u"ое", -1, 1),
        Among(u"ые", -1, 1),
        Among(u"ими", -1, 1),
        Among(u"ыми", -1, 1),
        Among(u"ей", -1, 1),
        Among(u"ий", -1, 1),
        Among(u"ой", -1, 1),
        Among(u"ый", -1, 1),
        Among(u"ем", -1, 1),
        Among(u"им", -1, 1),
        Among(u"ом", -1, 1),
        Among(u"ым", -1, 1),
        Among(u"его", -1, 1),
        Among(u"ого", -1, 1),
        Among(u"ему", -1, 1),
        Among(u"ому", -1, 1),
        Among(u"их", -1, 1),
        Among(u"ых", -1, 1),
        Among(u"ею", -1, 1),
        Among(u"ою", -1, 1),
        Among(u"ую", -1, 1),
        Among(u"юю", -1, 1),
        Among(u"ая", -1, 1),
        Among(u"яя", -1, 1)
    ]

    a_2 = [
        Among(u"ем", -1, 1),
        Among(u"нн", -1, 1),
        Among(u"вш", -1, 1),
        Among(u"ивш", 2, 2),
        Among(u"ывш", 2, 2),
        Among(u"щ", -1, 1),
        Among(u"ющ", 5, 1),
        Among(u"ующ", 6, 2)
    ]

    a_3 = [
        Among(u"сь", -1, 1),
        Among(u"ся", -1, 1)
    ]

    a_4 = [
        Among(u"ла", -1, 1),
        Among(u"ила", 0, 2),
        Among(u"ыла", 0, 2),
        Among(u"на", -1, 1),
        Among(u"ена", 3, 2),
        Among(u"ете", -1, 1),
        Among(u"ите", -1, 2),
        Among(u"йте", -1, 1),
        Among(u"ейте", 7, 2),
        Among(u"уйте", 7, 2),
        Among(u"ли", -1, 1),
        Among(u"или", 10, 2),
        Among(u"ыли", 10, 2),
        Among(u"й", -1, 1),
        Among(u"ей", 13, 2),
        Among(u"уй", 13, 2),
        Among(u"л", -1, 1),
        Among(u"ил", 16, 2),
        Among(u"ыл", 16, 2),
        Among(u"ем", -1, 1),
        Among(u"им", -1, 2),
        Among(u"ым", -1, 2),
        Among(u"н", -1, 1),
        Among(u"ен", 22, 2),
        Among(u"ло", -1, 1),
        Among(u"ило", 24, 2),
        Among(u"ыло", 24, 2),
        Among(u"но", -1, 1),
        Among(u"ено", 27, 2),
        Among(u"нно", 27, 1),
        Among(u"ет", -1, 1),
        Among(u"ует", 30, 2),
        Among(u"ит", -1, 2),
        Among(u"ыт", -1, 2),
        Among(u"ют", -1, 1),
        Among(u"уют", 34, 2),
        Among(u"ят", -1, 2),
        Among(u"ны", -1, 1),
        Among(u"ены", 37, 2),
        Among(u"ть", -1, 1),
        Among(u"ить", 39, 2),
        Among(u"ыть", 39, 2),
        Among(u"ешь", -1, 1),
        Among(u"ишь", -1, 2),
        Among(u"ю", -1, 2),
        Among(u"ую", 44, 2)
    ]

    a_5 = [
        Among(u"а", -1, 1),
        Among(u"ев", -1, 1),
        Among(u"ов", -1, 1),
        Among(u"е", -1, 1),
        Among(u"ие", 3, 1),
        Among(u"ье", 3, 1),
        Among(u"и", -1, 1),
        Among(u"еи", 6, 1),
        Among(u"ии", 6, 1),
        Among(u"ами", 6, 1),
        Among(u"ями", 6, 1),
        Among(u"иями", 10, 1),
        Among(u"й", -1, 1),
        Among(u"ей", 12, 1),
        Among(u"ией", 13, 1),
        Among(u"ий", 12, 1),
        Among(u"ой", 12, 1),
        Among(u"ам", -1, 1),
        Among(u"ем", -1, 1),
        Among(u"ием", 18, 1),
        Among(u"ом", -1, 1),
        Among(u"ям", -1, 1),
        Among(u"иям", 21, 1),
        Among(u"о", -1, 1),
        Among(u"у", -1, 1),
        Among(u"ах", -1, 1),
        Among(u"ях", -1, 1),
        Among(u"иях", 26, 1),
        Among(u"ы", -1, 1),
        Among(u"ь", -1, 1),
        Among(u"ю", -1, 1),
        Among(u"ию", 30, 1),
        Among(u"ью", 30, 1),
        Among(u"я", -1, 1),
        Among(u"ия", 33, 1),
        Among(u"ья", 33, 1)
    ]

    a_6 = [
        Among(u"ост", -1, 1),
        Among(u"ость", -1, 1)
    ]

    a_7 = [
        Among(u"ейше", -1, 1),
        Among(u"н", -1, 2),
        Among(u"ейш", -1, 1),
        Among(u"ь", -1, 3)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass


class lab4(BaseException): pass


class lab5(BaseException): pass


class lab6(BaseException): pass


class lab7(BaseException): pass


class lab8(BaseException): pass


class lab9(BaseException): pass


class lab10(BaseException): pass


class lab11(BaseException): pass



#-*- coding: utf-8 -*-
# Generated from german.sbl by Snowball 3.0.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class GermanStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from german.sbl by Snowball 3.0.1 - https://snowballstem.org/
    '''

    g_v = {u"a", u"e", u"i", u"o", u"u", u"y", u"ä", u"ö", u"ü"}

    g_et_ending = {u"d", u"f", u"g", u"k", u"l", u"m", u"n", u"r", u"s", u"t", u"U", u"z", u"ä"}

    g_s_ending = {u"b", u"d", u"f", u"g", u"h", u"k", u"l", u"m", u"n", u"r", u"t"}

    g_st_ending = {u"b", u"d", u"f", u"g", u"h", u"k", u"l", u"m", u"n", u"t"}

    I_x = 0
    I_p2 = 0
    I_p1 = 0

    def __r_prelude(self):
        v_1 = self.cursor
        while True:
            v_2 = self.cursor
            try:
                try:
                    while True:
                        v_3 = self.cursor
                        try:
                            if not self.in_grouping(GermanStemmer.g_v):
                                raise lab2()
                            self.bra = self.cursor
                            try:
                                v_4 = self.cursor
                                try:
                                    if not self.eq_s(u"u"):
                                        raise lab4()
                                    self.ket = self.cursor
                                    if not self.in_grouping(GermanStemmer.g_v):
                                        raise lab4()
                                    if not self.slice_from(u"U"):
                                        return False
                                    raise lab3()
                                except lab4: pass
                                self.cursor = v_4
                                if not self.eq_s(u"y"):
                                    raise lab2()
                                self.ket = self.cursor
                                if not self.in_grouping(GermanStemmer.g_v):
                                    raise lab2()
                                if not self.slice_from(u"Y"):
                                    return False
                            except lab3: pass
                            self.cursor = v_3
                            raise lab1()
                        except lab2: pass
                        self.cursor = v_3
                        if self.cursor >= self.limit:
                            raise lab0()
                        self.cursor += 1
                except lab1: pass
                continue
            except lab0: pass
            self.cursor = v_2
            break
        self.cursor = v_1
        while True:
            v_5 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(GermanStemmer.a_0)
                self.ket = self.cursor
                if among_var == 1:
                    if not self.slice_from(u"ss"):
                        return False
                elif among_var == 2:
                    if not self.slice_from(u"ä"):
                        return False
                elif among_var == 3:
                    if not self.slice_from(u"ö"):
                        return False
                elif among_var == 4:
                    if not self.slice_from(u"ü"):
                        return False
                elif among_var == 5:
                    if self.cursor >= self.limit:
                        raise lab5()
                    self.cursor += 1
                continue
            except lab5: pass
            self.cursor = v_5
            break
        return True

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        c = self.cursor + 3
        if c > self.limit:
            return False
        self.cursor = c
        self.I_x = self.cursor
        self.cursor = v_1
        if not self.go_out_grouping(GermanStemmer.g_v):
            return False
        self.cursor += 1
        if not self.go_in_grouping(GermanStemmer.g_v):
            return False
        self.cursor += 1
        self.I_p1 = self.cursor
        try:
            if self.I_p1 >= self.I_x:
                raise lab0()
            self.I_p1 = self.I_x
        except lab0: pass
        if not self.go_out_grouping(GermanStemmer.g_v):
            return False
        self.cursor += 1
        if not self.go_in_grouping(GermanStemmer.g_v):
            return False
        self.cursor += 1
        self.I_p2 = self.cursor
        return True

    def __r_postlude(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(GermanStemmer.a_1)
                self.ket = self.cursor
                if among_var == 1:
                    if not self.slice_from(u"y"):
                        return False
                elif among_var == 2:
                    if not self.slice_from(u"u"):
                        return False
                elif among_var == 3:
                    if not self.slice_from(u"a"):
                        return False
                elif among_var == 4:
                    if not self.slice_from(u"o"):
                        return False
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_R2(self):
        return self.I_p2 <= self.cursor

    def __r_standard_suffix(self):
        v_1 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(GermanStemmer.a_2)
            if among_var == 0:
                raise lab0()
            self.bra = self.cursor
            if not self.__r_R1():
                raise lab0()
            if among_var == 1:
                v_2 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"syst"):
                        raise lab1()
                    raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_2
                if not self.slice_del():
                    return False

            elif among_var == 2:
                if not self.slice_del():
                    return False

            elif among_var == 3:
                if not self.slice_del():
                    return False

                v_3 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    if not self.eq_s_b(u"s"):
                        self.cursor = self.limit - v_3
                        raise lab2()
                    self.bra = self.cursor
                    if not self.eq_s_b(u"nis"):
                        self.cursor = self.limit - v_3
                        raise lab2()
                    if not self.slice_del():
                        return False

                except lab2: pass
            elif among_var == 4:
                if not self.in_grouping_b(GermanStemmer.g_s_ending):
                    raise lab0()
                if not self.slice_del():
                    return False

            else:
                if not self.slice_from(u"l"):
                    return False
        except lab0: pass
        self.cursor = self.limit - v_1
        v_4 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(GermanStemmer.a_4)
            if among_var == 0:
                raise lab3()
            self.bra = self.cursor
            if not self.__r_R1():
                raise lab3()
            if among_var == 1:
                if not self.slice_del():
                    return False

            elif among_var == 2:
                if not self.in_grouping_b(GermanStemmer.g_st_ending):
                    raise lab3()
                c = self.cursor - 3
                if c < self.limit_backward:
                    raise lab3()
                self.cursor = c
                if not self.slice_del():
                    return False

            else:
                v_5 = self.limit - self.cursor
                if not self.in_grouping_b(GermanStemmer.g_et_ending):
                    raise lab3()
                self.cursor = self.limit - v_5
                v_6 = self.limit - self.cursor
                try:
                    if self.find_among_b(GermanStemmer.a_3) == 0:
                        raise lab4()
                    raise lab3()
                except lab4: pass
                self.cursor = self.limit - v_6
                if not self.slice_del():
                    return False

        except lab3: pass
        self.cursor = self.limit - v_4
        v_7 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(GermanStemmer.a_6)
            if among_var == 0:
                raise lab5()
            self.bra = self.cursor
            if not self.__r_R2():
                raise lab5()
            if among_var == 1:
                if not self.slice_del():
                    return False

                v_8 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    if not self.eq_s_b(u"ig"):
                        self.cursor = self.limit - v_8
                        raise lab6()
                    self.bra = self.cursor
                    v_9 = self.limit - self.cursor
                    try:
                        if not self.eq_s_b(u"e"):
                            raise lab7()
                        self.cursor = self.limit - v_8
                        raise lab6()
                    except lab7: pass
                    self.cursor = self.limit - v_9
                    if not self.__r_R2():
                        self.cursor = self.limit - v_8
                        raise lab6()
                    if not self.slice_del():
                        return False

                except lab6: pass
            elif among_var == 2:
                v_10 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"e"):
                        raise lab8()
                    raise lab5()
                except lab8: pass
                self.cursor = self.limit - v_10
                if not self.slice_del():
                    return False

            elif among_var == 3:
                if not self.slice_del():
                    return False

                v_11 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    try:
                        v_12 = self.limit - self.cursor
                        try:
                            if not self.eq_s_b(u"er"):
                                raise lab11()
                            raise lab10()
                        except lab11: pass
                        self.cursor = self.limit - v_12
                        if not self.eq_s_b(u"en"):
                            self.cursor = self.limit - v_11
                            raise lab9()
                    except lab10: pass
                    self.bra = self.cursor
                    if not self.__r_R1():
                        self.cursor = self.limit - v_11
                        raise lab9()
                    if not self.slice_del():
                        return False

                except lab9: pass
            else:
                if not self.slice_del():
                    return False

                v_13 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    if self.find_among_b(GermanStemmer.a_5) == 0:
                        self.cursor = self.limit - v_13
                        raise lab12()
                    self.bra = self.cursor
                    if not self.__r_R2():
                        self.cursor = self.limit - v_13
                        raise lab12()
                    if not self.slice_del():
                        return False

                except lab12: pass
        except lab5: pass
        self.cursor = self.limit - v_7
        return True

    def _stem(self):
        v_1 = self.cursor
        self.__r_prelude()
        self.cursor = v_1
        v_2 = self.cursor
        self.__r_mark_regions()
        self.cursor = v_2
        self.limit_backward = self.cursor
        self.cursor = self.limit
        self.__r_standard_suffix()
        self.cursor = self.limit_backward
        v_4 = self.cursor
        self.__r_postlude()
        self.cursor = v_4
        return True

    a_0 = [
        Among(u"", -1, 5),
        Among(u"ae", 0, 2),
        Among(u"oe", 0, 3),
        Among(u"qu", 0, -1),
        Among(u"ue", 0, 4),
        Among(u"ß", 0, 1)
    ]

    a_1 = [
        Among(u"", -1, 5),
        Among(u"U", 0, 2),
        Among(u"Y", 0, 1),
        Among(u"ä", 0, 3),
        Among(u"ö", 0, 4),
        Among(u"ü", 0, 2)
    ]

    a_2 = [
        Among(u"e", -1, 3),
        Among(u"em", -1, 1),
        Among(u"en", -1, 3),
        Among(u"erinnen", 2, 2),
        Among(u"erin", -1, 2),
        Among(u"ln", -1, 5),
        Among(u"ern", -1, 2),
        Among(u"er", -1, 2),
        Among(u"s", -1, 4),
        Among(u"es", 8, 3),
        Among(u"lns", 8, 5)
    ]

    a_3 = [
        Among(u"tick", -1, -1),
        Among(u"plan", -1, -1),
        Among(u"geordn", -1, -1),
        Among(u"intern", -1, -1),
        Among(u"tr", -1, -1)
    ]

    a_4 = [
        Among(u"en", -1, 1),
        Among(u"er", -1, 1),
        Among(u"et", -1, 3),
        Among(u"st", -1, 2),
        Among(u"est", 3, 1)
    ]

    a_5 = [
        Among(u"ig", -1, 1),
        Among(u"lich", -1, 1)
    ]

    a_6 = [
        Among(u"end", -1, 1),
        Among(u"ig", -1, 2),
        Among(u"ung", -1, 1),
        Among(u"lich", -1, 3),
        Among(u"isch", -1, 2),
        Among(u"ik", -1, 2),
        Among(u"heit", -1, 3),
        Among(u"keit", -1, 4)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass


class lab4(BaseException): pass


class lab5(BaseException): pass


class lab6(BaseException): pass


class lab7(BaseException): pass


class lab8(BaseException): pass


class lab9(BaseException): pass


class lab10(BaseException): pass


class lab11(BaseException): pass


class lab12(BaseException): pass



#-*- coding: utf-8 -*-
# Generated from irish.sbl by Snowball 3.0.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class IrishStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from irish.sbl by Snowball 3.0.1 - https://snowballstem.org/
    '''

    g_v = {u"a", u"e", u"i", u"o", u"u", u"á", u"é", u"í", u"ó", u"ú"}

    I_p2 = 0
    I_p1 = 0
    I_pV = 0

    def __r_mark_regions(self):
        self.I_pV = self.limit
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            if not self.go_out_grouping(IrishStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_pV = self.cursor
            if not self.go_in_grouping(IrishStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p1 = self.cursor
            if not self.go_out_grouping(IrishStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(IrishStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab0: pass
        self.cursor = v_1
        return True

    def __r_initial_morph(self):
        self.bra = self.cursor
        among_var = self.find_among(IrishStemmer.a_0)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if among_var == 1:
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.slice_from(u"f"):
                return False
        elif among_var == 3:
            if not self.slice_from(u"s"):
                return False
        elif among_var == 4:
            if not self.slice_from(u"b"):
                return False
        elif among_var == 5:
            if not self.slice_from(u"c"):
                return False
        elif among_var == 6:
            if not self.slice_from(u"d"):
                return False
        elif among_var == 7:
            if not self.slice_from(u"g"):
                return False
        elif among_var == 8:
            if not self.slice_from(u"p"):
                return False
        elif among_var == 9:
            if not self.slice_from(u"t"):
                return False
        else:
            if not self.slice_from(u"m"):
                return False
        return True

    def __r_RV(self):
        return self.I_pV <= self.cursor

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_R2(self):
        return self.I_p2 <= self.cursor

    def __r_noun_sfx(self):
        self.ket = self.cursor
        among_var = self.find_among_b(IrishStemmer.a_1)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R1():
                return False
            if not self.slice_del():
                return False

        else:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

        return True

    def __r_deriv(self):
        self.ket = self.cursor
        among_var = self.find_among_b(IrishStemmer.a_2)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.slice_from(u"arc"):
                return False
        elif among_var == 3:
            if not self.slice_from(u"gin"):
                return False
        elif among_var == 4:
            if not self.slice_from(u"graf"):
                return False
        elif among_var == 5:
            if not self.slice_from(u"paite"):
                return False
        else:
            if not self.slice_from(u"óid"):
                return False
        return True

    def __r_verb_sfx(self):
        self.ket = self.cursor
        among_var = self.find_among_b(IrishStemmer.a_3)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_RV():
                return False
            if not self.slice_del():
                return False

        else:
            if not self.__r_R1():
                return False
            if not self.slice_del():
                return False

        return True

    def _stem(self):
        v_1 = self.cursor
        self.__r_initial_morph()
        self.cursor = v_1
        self.__r_mark_regions()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_3 = self.limit - self.cursor
        self.__r_noun_sfx()
        self.cursor = self.limit - v_3
        v_4 = self.limit - self.cursor
        self.__r_deriv()
        self.cursor = self.limit - v_4
        v_5 = self.limit - self.cursor
        self.__r_verb_sfx()
        self.cursor = self.limit - v_5
        self.cursor = self.limit_backward
        return True

    a_0 = [
        Among(u"b'", -1, 1),
        Among(u"bh", -1, 4),
        Among(u"bhf", 1, 2),
        Among(u"bp", -1, 8),
        Among(u"ch", -1, 5),
        Among(u"d'", -1, 1),
        Among(u"d'fh", 5, 2),
        Among(u"dh", -1, 6),
        Among(u"dt", -1, 9),
        Among(u"fh", -1, 2),
        Among(u"gc", -1, 5),
        Among(u"gh", -1, 7),
        Among(u"h-", -1, 1),
        Among(u"m'", -1, 1),
        Among(u"mb", -1, 4),
        Among(u"mh", -1, 10),
        Among(u"n-", -1, 1),
        Among(u"nd", -1, 6),
        Among(u"ng", -1, 7),
        Among(u"ph", -1, 8),
        Among(u"sh", -1, 3),
        Among(u"t-", -1, 1),
        Among(u"th", -1, 9),
        Among(u"ts", -1, 3)
    ]

    a_1 = [
        Among(u"íochta", -1, 1),
        Among(u"aíochta", 0, 1),
        Among(u"ire", -1, 2),
        Among(u"aire", 2, 2),
        Among(u"abh", -1, 1),
        Among(u"eabh", 4, 1),
        Among(u"ibh", -1, 1),
        Among(u"aibh", 6, 1),
        Among(u"amh", -1, 1),
        Among(u"eamh", 8, 1),
        Among(u"imh", -1, 1),
        Among(u"aimh", 10, 1),
        Among(u"íocht", -1, 1),
        Among(u"aíocht", 12, 1),
        Among(u"irí", -1, 2),
        Among(u"airí", 14, 2)
    ]

    a_2 = [
        Among(u"óideacha", -1, 6),
        Among(u"patacha", -1, 5),
        Among(u"achta", -1, 1),
        Among(u"arcachta", 2, 2),
        Among(u"eachta", 2, 1),
        Among(u"grafaíochta", -1, 4),
        Among(u"paite", -1, 5),
        Among(u"ach", -1, 1),
        Among(u"each", 7, 1),
        Among(u"óideach", 8, 6),
        Among(u"gineach", 8, 3),
        Among(u"patach", 7, 5),
        Among(u"grafaíoch", -1, 4),
        Among(u"pataigh", -1, 5),
        Among(u"óidigh", -1, 6),
        Among(u"achtúil", -1, 1),
        Among(u"eachtúil", 15, 1),
        Among(u"gineas", -1, 3),
        Among(u"ginis", -1, 3),
        Among(u"acht", -1, 1),
        Among(u"arcacht", 19, 2),
        Among(u"eacht", 19, 1),
        Among(u"grafaíocht", -1, 4),
        Among(u"arcachtaí", -1, 2),
        Among(u"grafaíochtaí", -1, 4)
    ]

    a_3 = [
        Among(u"imid", -1, 1),
        Among(u"aimid", 0, 1),
        Among(u"ímid", -1, 1),
        Among(u"aímid", 2, 1),
        Among(u"adh", -1, 2),
        Among(u"eadh", 4, 2),
        Among(u"faidh", -1, 1),
        Among(u"fidh", -1, 1),
        Among(u"áil", -1, 2),
        Among(u"ain", -1, 2),
        Among(u"tear", -1, 2),
        Among(u"tar", -1, 2)
    ]


class lab0(BaseException): pass



#-*- coding: utf-8 -*-
# Generated from dutch.sbl by Snowball 3.0.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class DutchStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from dutch.sbl by Snowball 3.0.1 - https://snowballstem.org/
    '''

    g_E = {u"e", u"ë", u"é", u"è", u"ê"}

    g_AIOU = {u"a", u"ä", u"á", u"à", u"â", u"i", u"ï", u"í", u"ì", u"î", u"o", u"ö", u"ó", u"ò", u"ô", u"u", u"ü", u"ú", u"ù", u"û"}

    g_AEIOU = {u"a", u"ä", u"á", u"à", u"â", u"e", u"ë", u"é", u"è", u"ê", u"i", u"ï", u"í", u"ì", u"î", u"o", u"ö", u"ó", u"ò", u"ô", u"u", u"ü", u"ú", u"ù", u"û"}

    g_v = {u"a", u"ä", u"á", u"à", u"â", u"e", u"ë", u"é", u"è", u"ê", u"i", u"ï", u"í", u"ì", u"î", u"o", u"ö", u"ó", u"ò", u"ô", u"u", u"ü", u"ú", u"ù", u"û", u"y"}

    g_v_WX = {u"a", u"ä", u"á", u"à", u"â", u"e", u"ë", u"é", u"è", u"ê", u"i", u"ï", u"í", u"ì", u"î", u"o", u"ö", u"ó", u"ò", u"ô", u"u", u"ü", u"ú", u"ù", u"û", u"y", u"w", u"x"}

    B_GE_removed = False
    B_stemmed = False
    I_p2 = 0
    I_p1 = 0
    S_ch = ""

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_R2(self):
        return self.I_p2 <= self.cursor

    def __r_V(self):
        v_1 = self.limit - self.cursor
        try:
            v_2 = self.limit - self.cursor
            try:
                if not self.in_grouping_b(DutchStemmer.g_v):
                    raise lab1()
                raise lab0()
            except lab1: pass
            self.cursor = self.limit - v_2
            if not self.eq_s_b(u"ij"):
                return False
        except lab0: pass
        self.cursor = self.limit - v_1
        return True

    def __r_VX(self):
        v_1 = self.limit - self.cursor
        if self.cursor <= self.limit_backward:
            return False
        self.cursor -= 1
        try:
            v_2 = self.limit - self.cursor
            try:
                if not self.in_grouping_b(DutchStemmer.g_v):
                    raise lab1()
                raise lab0()
            except lab1: pass
            self.cursor = self.limit - v_2
            if not self.eq_s_b(u"ij"):
                return False
        except lab0: pass
        self.cursor = self.limit - v_1
        return True

    def __r_C(self):
        v_1 = self.limit - self.cursor
        v_2 = self.limit - self.cursor
        try:
            if not self.eq_s_b(u"ij"):
                raise lab0()
            return False
        except lab0: pass
        self.cursor = self.limit - v_2
        if not self.out_grouping_b(DutchStemmer.g_v):
            return False
        self.cursor = self.limit - v_1
        return True

    def __r_lengthen_V(self):
        v_1 = self.limit - self.cursor
        try:
            if not self.out_grouping_b(DutchStemmer.g_v_WX):
                raise lab0()
            self.ket = self.cursor
            among_var = self.find_among_b(DutchStemmer.a_0)
            if among_var == 0:
                raise lab0()
            self.bra = self.cursor
            if among_var == 1:
                v_2 = self.limit - self.cursor
                try:
                    v_3 = self.limit - self.cursor
                    try:
                        if not self.out_grouping_b(DutchStemmer.g_AEIOU):
                            raise lab2()
                        raise lab1()
                    except lab2: pass
                    self.cursor = self.limit - v_3
                    if self.cursor > self.limit_backward:
                        raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_2
                self.S_ch = self.slice_to()
                if self.S_ch == '':
                    return False
                c = self.cursor
                self.insert(self.cursor, self.cursor, self.S_ch)
                self.cursor = c
            elif among_var == 2:
                v_4 = self.limit - self.cursor
                try:
                    v_5 = self.limit - self.cursor
                    try:
                        if not self.out_grouping_b(DutchStemmer.g_AEIOU):
                            raise lab4()
                        raise lab3()
                    except lab4: pass
                    self.cursor = self.limit - v_5
                    if self.cursor > self.limit_backward:
                        raise lab0()
                except lab3: pass
                v_6 = self.limit - self.cursor
                try:
                    try:
                        v_7 = self.limit - self.cursor
                        try:
                            if not self.in_grouping_b(DutchStemmer.g_AIOU):
                                raise lab7()
                            raise lab6()
                        except lab7: pass
                        self.cursor = self.limit - v_7
                        if not self.in_grouping_b(DutchStemmer.g_E):
                            raise lab5()
                        if self.cursor > self.limit_backward:
                            raise lab5()
                    except lab6: pass
                    raise lab0()
                except lab5: pass
                self.cursor = self.limit - v_6
                v_8 = self.limit - self.cursor
                try:
                    if self.cursor <= self.limit_backward:
                        raise lab8()
                    self.cursor -= 1
                    if not self.in_grouping_b(DutchStemmer.g_AIOU):
                        raise lab8()
                    if not self.out_grouping_b(DutchStemmer.g_AEIOU):
                        raise lab8()
                    raise lab0()
                except lab8: pass
                self.cursor = self.limit - v_8
                self.cursor = self.limit - v_4
                self.S_ch = self.slice_to()
                if self.S_ch == '':
                    return False
                c = self.cursor
                self.insert(self.cursor, self.cursor, self.S_ch)
                self.cursor = c
            elif among_var == 3:
                if not self.slice_from(u"eëe"):
                    return False
            else:
                if not self.slice_from(u"iee"):
                    return False
        except lab0: pass
        self.cursor = self.limit - v_1
        return True

    def __r_Step_1(self):
        self.ket = self.cursor
        among_var = self.find_among_b(DutchStemmer.a_1)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.slice_del():
                return False

        elif among_var == 2:
            if not self.__r_R1():
                return False
            v_1 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"t"):
                    raise lab0()
                if not self.__r_R1():
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_1
            if not self.__r_C():
                return False
            if not self.slice_del():
                return False

        elif among_var == 3:
            if not self.__r_R1():
                return False
            if not self.slice_from(u"ie"):
                return False
        elif among_var == 4:
            try:
                v_2 = self.limit - self.cursor
                try:
                    v_3 = self.limit - self.cursor
                    if not self.eq_s_b(u"ar"):
                        raise lab2()
                    if not self.__r_R1():
                        raise lab2()
                    if not self.__r_C():
                        raise lab2()
                    self.cursor = self.limit - v_3
                    if not self.slice_del():
                        return False

                    self.__r_lengthen_V()
                    raise lab1()
                except lab2: pass
                self.cursor = self.limit - v_2
                try:
                    v_4 = self.limit - self.cursor
                    if not self.eq_s_b(u"er"):
                        raise lab3()
                    if not self.__r_R1():
                        raise lab3()
                    if not self.__r_C():
                        raise lab3()
                    self.cursor = self.limit - v_4
                    if not self.slice_del():
                        return False

                    raise lab1()
                except lab3: pass
                self.cursor = self.limit - v_2
                if not self.__r_R1():
                    return False
                if not self.__r_C():
                    return False
                if not self.slice_from(u"e"):
                    return False
            except lab1: pass
        elif among_var == 5:
            if not self.__r_R1():
                return False
            if not self.slice_from(u"é"):
                return False
        elif among_var == 6:
            if not self.__r_R1():
                return False
            if not self.__r_V():
                return False
            if not self.slice_from(u"au"):
                return False
        elif among_var == 7:
            try:
                v_5 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"hed"):
                        raise lab5()
                    if not self.__r_R1():
                        raise lab5()
                    self.bra = self.cursor
                    if not self.slice_from(u"heid"):
                        return False
                    raise lab4()
                except lab5: pass
                self.cursor = self.limit - v_5
                try:
                    if not self.eq_s_b(u"nd"):
                        raise lab6()
                    if not self.slice_del():
                        return False

                    raise lab4()
                except lab6: pass
                self.cursor = self.limit - v_5
                try:
                    if not self.eq_s_b(u"d"):
                        raise lab7()
                    if not self.__r_R1():
                        raise lab7()
                    if not self.__r_C():
                        raise lab7()
                    self.bra = self.cursor
                    if not self.slice_del():
                        return False

                    raise lab4()
                except lab7: pass
                self.cursor = self.limit - v_5
                try:
                    try:
                        v_6 = self.limit - self.cursor
                        try:
                            if not self.eq_s_b(u"i"):
                                raise lab10()
                            raise lab9()
                        except lab10: pass
                        self.cursor = self.limit - v_6
                        if not self.eq_s_b(u"j"):
                            raise lab8()
                    except lab9: pass
                    if not self.__r_V():
                        raise lab8()
                    if not self.slice_del():
                        return False

                    raise lab4()
                except lab8: pass
                self.cursor = self.limit - v_5
                if not self.__r_R1():
                    return False
                if not self.__r_C():
                    return False
                if not self.slice_del():
                    return False

                self.__r_lengthen_V()
            except lab4: pass
        else:
            if not self.slice_from(u"nd"):
                return False
        return True

    def __r_Step_2(self):
        self.ket = self.cursor
        among_var = self.find_among_b(DutchStemmer.a_2)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            try:
                v_1 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"'t"):
                        raise lab1()
                    self.bra = self.cursor
                    if not self.slice_del():
                        return False

                    raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_1
                try:
                    if not self.eq_s_b(u"et"):
                        raise lab2()
                    self.bra = self.cursor
                    if not self.__r_R1():
                        raise lab2()
                    if not self.__r_C():
                        raise lab2()
                    if not self.slice_del():
                        return False

                    raise lab0()
                except lab2: pass
                self.cursor = self.limit - v_1
                try:
                    if not self.eq_s_b(u"rnt"):
                        raise lab3()
                    self.bra = self.cursor
                    if not self.slice_from(u"rn"):
                        return False
                    raise lab0()
                except lab3: pass
                self.cursor = self.limit - v_1
                try:
                    if not self.eq_s_b(u"t"):
                        raise lab4()
                    self.bra = self.cursor
                    if not self.__r_R1():
                        raise lab4()
                    if not self.__r_VX():
                        raise lab4()
                    if not self.slice_del():
                        return False

                    raise lab0()
                except lab4: pass
                self.cursor = self.limit - v_1
                try:
                    if not self.eq_s_b(u"ink"):
                        raise lab5()
                    self.bra = self.cursor
                    if not self.slice_from(u"ing"):
                        return False
                    raise lab0()
                except lab5: pass
                self.cursor = self.limit - v_1
                try:
                    if not self.eq_s_b(u"mp"):
                        raise lab6()
                    self.bra = self.cursor
                    if not self.slice_from(u"m"):
                        return False
                    raise lab0()
                except lab6: pass
                self.cursor = self.limit - v_1
                try:
                    if not self.eq_s_b(u"'"):
                        raise lab7()
                    self.bra = self.cursor
                    if not self.__r_R1():
                        raise lab7()
                    if not self.slice_del():
                        return False

                    raise lab0()
                except lab7: pass
                self.cursor = self.limit - v_1
                self.bra = self.cursor
                if not self.__r_R1():
                    return False
                if not self.__r_C():
                    return False
                if not self.slice_del():
                    return False

            except lab0: pass
        elif among_var == 2:
            if not self.__r_R1():
                return False
            if not self.slice_from(u"g"):
                return False
        elif among_var == 3:
            if not self.__r_R1():
                return False
            if not self.slice_from(u"lijk"):
                return False
        elif among_var == 4:
            if not self.__r_R1():
                return False
            if not self.slice_from(u"isch"):
                return False
        elif among_var == 5:
            if not self.__r_R1():
                return False
            if not self.__r_C():
                return False
            if not self.slice_del():
                return False

        elif among_var == 6:
            if not self.__r_R1():
                return False
            if not self.slice_from(u"t"):
                return False
        elif among_var == 7:
            if not self.__r_R1():
                return False
            if not self.slice_from(u"s"):
                return False
        elif among_var == 8:
            if not self.__r_R1():
                return False
            if not self.slice_from(u"r"):
                return False
        elif among_var == 9:
            if not self.__r_R1():
                return False
            if not self.slice_del():
                return False

            self.insert(self.cursor, self.cursor, u"l")
            self.__r_lengthen_V()
        elif among_var == 10:
            if not self.__r_R1():
                return False
            if not self.__r_C():
                return False
            if not self.slice_del():
                return False

            self.insert(self.cursor, self.cursor, u"en")
            self.__r_lengthen_V()
        else:
            if not self.__r_R1():
                return False
            if not self.__r_C():
                return False
            if not self.slice_from(u"ief"):
                return False
        return True

    def __r_Step_3(self):
        self.ket = self.cursor
        among_var = self.find_among_b(DutchStemmer.a_3)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R1():
                return False
            if not self.slice_from(u"eer"):
                return False
        elif among_var == 2:
            if not self.__r_R1():
                return False
            if not self.slice_del():
                return False

            self.__r_lengthen_V()
        elif among_var == 3:
            if not self.__r_R1():
                return False
            if not self.slice_del():
                return False

        elif among_var == 4:
            if not self.slice_from(u"r"):
                return False
        elif among_var == 5:
            try:
                v_1 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"ild"):
                        raise lab1()
                    if not self.slice_from(u"er"):
                        return False
                    raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_1
                if not self.__r_R1():
                    return False
                if not self.slice_del():
                    return False

                self.__r_lengthen_V()
            except lab0: pass
        elif among_var == 6:
            if not self.__r_R1():
                return False
            if not self.__r_C():
                return False
            if not self.slice_from(u"aar"):
                return False
        elif among_var == 7:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

            self.insert(self.cursor, self.cursor, u"f")
            self.__r_lengthen_V()
        elif among_var == 8:
            if not self.__r_R2():
                return False
            if not self.slice_del():
                return False

            self.insert(self.cursor, self.cursor, u"g")
            self.__r_lengthen_V()
        elif among_var == 9:
            if not self.__r_R1():
                return False
            if not self.__r_C():
                return False
            if not self.slice_from(u"t"):
                return False
        else:
            if not self.__r_R1():
                return False
            if not self.__r_C():
                return False
            if not self.slice_from(u"d"):
                return False
        return True

    def __r_Step_4(self):
        try:
            v_1 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                among_var = self.find_among_b(DutchStemmer.a_4)
                if among_var == 0:
                    raise lab1()
                self.bra = self.cursor
                if among_var == 1:
                    if not self.__r_R1():
                        raise lab1()
                    if not self.slice_from(u"ie"):
                        return False
                elif among_var == 2:
                    if not self.__r_R1():
                        raise lab1()
                    if not self.slice_from(u"eer"):
                        return False
                elif among_var == 3:
                    if not self.__r_R1():
                        raise lab1()
                    if not self.slice_del():
                        return False

                elif among_var == 4:
                    if not self.__r_R1():
                        raise lab1()
                    if not self.__r_V():
                        raise lab1()
                    if not self.slice_from(u"n"):
                        return False
                elif among_var == 5:
                    if not self.__r_R1():
                        raise lab1()
                    if not self.__r_V():
                        raise lab1()
                    if not self.slice_from(u"l"):
                        return False
                elif among_var == 6:
                    if not self.__r_R1():
                        raise lab1()
                    if not self.__r_V():
                        raise lab1()
                    if not self.slice_from(u"r"):
                        return False
                elif among_var == 7:
                    if not self.__r_R1():
                        raise lab1()
                    if not self.slice_from(u"teer"):
                        return False
                elif among_var == 8:
                    if not self.__r_R1():
                        raise lab1()
                    if not self.slice_from(u"lijk"):
                        return False
                else:
                    if not self.__r_R1():
                        raise lab1()
                    if not self.__r_C():
                        raise lab1()
                    if not self.slice_del():
                        return False

                    self.__r_lengthen_V()
                raise lab0()
            except lab1: pass
            self.cursor = self.limit - v_1
            self.ket = self.cursor
            if self.find_among_b(DutchStemmer.a_5) == 0:
                return False
            self.bra = self.cursor
            if not self.__r_R1():
                return False
            v_2 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"inn"):
                    raise lab2()
                if self.cursor > self.limit_backward:
                    raise lab2()
                return False
            except lab2: pass
            self.cursor = self.limit - v_2
            if not self.__r_C():
                return False
            if not self.slice_del():
                return False

            self.__r_lengthen_V()
        except lab0: pass
        return True

    def __r_Step_7(self):
        self.ket = self.cursor
        among_var = self.find_among_b(DutchStemmer.a_6)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.slice_from(u"k"):
                return False
        elif among_var == 2:
            if not self.slice_from(u"f"):
                return False
        else:
            if not self.slice_from(u"p"):
                return False
        return True

    def __r_Step_6(self):
        self.ket = self.cursor
        among_var = self.find_among_b(DutchStemmer.a_7)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.slice_from(u"b"):
                return False
        elif among_var == 2:
            if not self.slice_from(u"c"):
                return False
        elif among_var == 3:
            if not self.slice_from(u"d"):
                return False
        elif among_var == 4:
            if not self.slice_from(u"f"):
                return False
        elif among_var == 5:
            if not self.slice_from(u"g"):
                return False
        elif among_var == 6:
            if not self.slice_from(u"h"):
                return False
        elif among_var == 7:
            if not self.slice_from(u"j"):
                return False
        elif among_var == 8:
            if not self.slice_from(u"k"):
                return False
        elif among_var == 9:
            if not self.slice_from(u"l"):
                return False
        elif among_var == 10:
            if not self.slice_from(u"m"):
                return False
        elif among_var == 11:
            v_1 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"i"):
                    raise lab0()
                if self.cursor > self.limit_backward:
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_1
            if not self.slice_from(u"n"):
                return False
        elif among_var == 12:
            if not self.slice_from(u"p"):
                return False
        elif among_var == 13:
            if not self.slice_from(u"q"):
                return False
        elif among_var == 14:
            if not self.slice_from(u"r"):
                return False
        elif among_var == 15:
            if not self.slice_from(u"s"):
                return False
        elif among_var == 16:
            if not self.slice_from(u"t"):
                return False
        elif among_var == 17:
            if not self.slice_from(u"v"):
                return False
        elif among_var == 18:
            if not self.slice_from(u"w"):
                return False
        elif among_var == 19:
            if not self.slice_from(u"x"):
                return False
        else:
            if not self.slice_from(u"z"):
                return False
        return True

    def __r_Step_1c(self):
        self.ket = self.cursor
        among_var = self.find_among_b(DutchStemmer.a_8)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if not self.__r_C():
            return False
        if among_var == 1:
            v_1 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"n"):
                    raise lab0()
                if not self.__r_R1():
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                v_2 = self.limit - self.cursor
                try:
                    if not self.eq_s_b(u"in"):
                        raise lab2()
                    if self.cursor > self.limit_backward:
                        raise lab2()
                    if not self.slice_from(u"n"):
                        return False
                    raise lab1()
                except lab2: pass
                self.cursor = self.limit - v_2
                if not self.slice_del():
                    return False

            except lab1: pass
        else:
            v_3 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"h"):
                    raise lab3()
                if not self.__r_R1():
                    raise lab3()
                return False
            except lab3: pass
            self.cursor = self.limit - v_3
            v_4 = self.limit - self.cursor
            try:
                if not self.eq_s_b(u"en"):
                    raise lab4()
                if self.cursor > self.limit_backward:
                    raise lab4()
                return False
            except lab4: pass
            self.cursor = self.limit - v_4
            if not self.slice_del():
                return False

        return True

    def __r_Lose_prefix(self):
        self.bra = self.cursor
        if not self.eq_s(u"ge"):
            return False
        self.ket = self.cursor
        v_1 = self.cursor
        c = self.cursor + 3
        if c > self.limit:
            return False
        self.cursor = c
        self.cursor = v_1
        v_2 = self.cursor
        try:
            while True:
                v_3 = self.cursor
                try:
                    try:
                        v_4 = self.cursor
                        try:
                            if not self.eq_s(u"ij"):
                                raise lab3()
                            raise lab2()
                        except lab3: pass
                        self.cursor = v_4
                        if not self.in_grouping(DutchStemmer.g_v):
                            raise lab1()
                    except lab2: pass
                    raise lab0()
                except lab1: pass
                self.cursor = v_3
                if self.cursor >= self.limit:
                    return False
                self.cursor += 1
        except lab0: pass
        while True:
            v_5 = self.cursor
            try:
                try:
                    v_6 = self.cursor
                    try:
                        if not self.eq_s(u"ij"):
                            raise lab6()
                        raise lab5()
                    except lab6: pass
                    self.cursor = v_6
                    if not self.in_grouping(DutchStemmer.g_v):
                        raise lab4()
                except lab5: pass
                continue
            except lab4: pass
            self.cursor = v_5
            break
        try:
            if self.cursor < self.limit:
                raise lab7()
            return False
        except lab7: pass
        self.cursor = v_2
        among_var = self.find_among(DutchStemmer.a_9)
        if among_var == 1:
            return False
        self.B_GE_removed = True
        if not self.slice_del():
            return False

        v_8 = self.cursor
        try:
            self.bra = self.cursor
            among_var = self.find_among(DutchStemmer.a_10)
            if among_var == 0:
                raise lab8()
            self.ket = self.cursor
            if among_var == 1:
                if not self.slice_from(u"e"):
                    return False
            else:
                if not self.slice_from(u"i"):
                    return False
        except lab8: pass
        self.cursor = v_8
        return True

    def __r_Lose_infix(self):
        if self.cursor >= self.limit:
            return False
        self.cursor += 1
        try:
            while True:
                try:
                    self.bra = self.cursor
                    if not self.eq_s(u"ge"):
                        raise lab1()
                    self.ket = self.cursor
                    raise lab0()
                except lab1: pass
                if self.cursor >= self.limit:
                    return False
                self.cursor += 1
        except lab0: pass
        v_2 = self.cursor
        c = self.cursor + 3
        if c > self.limit:
            return False
        self.cursor = c
        self.cursor = v_2
        v_3 = self.cursor
        try:
            while True:
                v_4 = self.cursor
                try:
                    try:
                        v_5 = self.cursor
                        try:
                            if not self.eq_s(u"ij"):
                                raise lab5()
                            raise lab4()
                        except lab5: pass
                        self.cursor = v_5
                        if not self.in_grouping(DutchStemmer.g_v):
                            raise lab3()
                    except lab4: pass
                    raise lab2()
                except lab3: pass
                self.cursor = v_4
                if self.cursor >= self.limit:
                    return False
                self.cursor += 1
        except lab2: pass
        while True:
            v_6 = self.cursor
            try:
                try:
                    v_7 = self.cursor
                    try:
                        if not self.eq_s(u"ij"):
                            raise lab8()
                        raise lab7()
                    except lab8: pass
                    self.cursor = v_7
                    if not self.in_grouping(DutchStemmer.g_v):
                        raise lab6()
                except lab7: pass
                continue
            except lab6: pass
            self.cursor = v_6
            break
        try:
            if self.cursor < self.limit:
                raise lab9()
            return False
        except lab9: pass
        self.cursor = v_3
        self.B_GE_removed = True
        if not self.slice_del():
            return False

        v_9 = self.cursor
        try:
            self.bra = self.cursor
            among_var = self.find_among(DutchStemmer.a_11)
            if among_var == 0:
                raise lab10()
            self.ket = self.cursor
            if among_var == 1:
                if not self.slice_from(u"e"):
                    return False
            else:
                if not self.slice_from(u"i"):
                    return False
        except lab10: pass
        self.cursor = v_9
        return True

    def __r_measure(self):
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            while True:
                try:
                    if not self.out_grouping(DutchStemmer.g_v):
                        raise lab1()
                    continue
                except lab1: pass
                break
            v_3 = 1
            while True:
                v_4 = self.cursor
                try:
                    try:
                        v_5 = self.cursor
                        try:
                            if not self.eq_s(u"ij"):
                                raise lab4()
                            raise lab3()
                        except lab4: pass
                        self.cursor = v_5
                        if not self.in_grouping(DutchStemmer.g_v):
                            raise lab2()
                    except lab3: pass
                    v_3 -= 1
                    continue
                except lab2: pass
                self.cursor = v_4
                break
            if v_3 > 0:
                raise lab0()
            if not self.out_grouping(DutchStemmer.g_v):
                raise lab0()
            self.I_p1 = self.cursor
            while True:
                try:
                    if not self.out_grouping(DutchStemmer.g_v):
                        raise lab5()
                    continue
                except lab5: pass
                break
            v_7 = 1
            while True:
                v_8 = self.cursor
                try:
                    try:
                        v_9 = self.cursor
                        try:
                            if not self.eq_s(u"ij"):
                                raise lab8()
                            raise lab7()
                        except lab8: pass
                        self.cursor = v_9
                        if not self.in_grouping(DutchStemmer.g_v):
                            raise lab6()
                    except lab7: pass
                    v_7 -= 1
                    continue
                except lab6: pass
                self.cursor = v_8
                break
            if v_7 > 0:
                raise lab0()
            if not self.out_grouping(DutchStemmer.g_v):
                raise lab0()
            self.I_p2 = self.cursor
        except lab0: pass
        self.cursor = v_1
        return True

    def _stem(self):
        self.B_stemmed = False
        self.__r_measure()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_1 = self.limit - self.cursor
        try:
            if not self.__r_Step_1():
                raise lab0()
            self.B_stemmed = True
        except lab0: pass
        self.cursor = self.limit - v_1
        v_2 = self.limit - self.cursor
        try:
            if not self.__r_Step_2():
                raise lab1()
            self.B_stemmed = True
        except lab1: pass
        self.cursor = self.limit - v_2
        v_3 = self.limit - self.cursor
        try:
            if not self.__r_Step_3():
                raise lab2()
            self.B_stemmed = True
        except lab2: pass
        self.cursor = self.limit - v_3
        v_4 = self.limit - self.cursor
        try:
            if not self.__r_Step_4():
                raise lab3()
            self.B_stemmed = True
        except lab3: pass
        self.cursor = self.limit - v_4
        self.cursor = self.limit_backward
        self.B_GE_removed = False
        v_5 = self.cursor
        try:
            v_6 = self.cursor
            if not self.__r_Lose_prefix():
                raise lab4()
            self.cursor = v_6
            self.__r_measure()
        except lab4: pass
        self.cursor = v_5
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_7 = self.limit - self.cursor
        try:
            if not self.B_GE_removed:
                raise lab5()
            self.B_stemmed = True
            if not self.__r_Step_1c():
                raise lab5()
        except lab5: pass
        self.cursor = self.limit - v_7
        self.cursor = self.limit_backward
        self.B_GE_removed = False
        v_8 = self.cursor
        try:
            v_9 = self.cursor
            if not self.__r_Lose_infix():
                raise lab6()
            self.cursor = v_9
            self.__r_measure()
        except lab6: pass
        self.cursor = v_8
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_10 = self.limit - self.cursor
        try:
            if not self.B_GE_removed:
                raise lab7()
            self.B_stemmed = True
            if not self.__r_Step_1c():
                raise lab7()
        except lab7: pass
        self.cursor = self.limit - v_10
        self.cursor = self.limit_backward
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_11 = self.limit - self.cursor
        try:
            if not self.__r_Step_7():
                raise lab8()
            self.B_stemmed = True
        except lab8: pass
        self.cursor = self.limit - v_11
        v_12 = self.limit - self.cursor
        try:
            if not self.B_stemmed:
                raise lab9()
            if not self.__r_Step_6():
                raise lab9()
        except lab9: pass
        self.cursor = self.limit - v_12
        self.cursor = self.limit_backward
        return True

    a_0 = [
        Among(u"a", -1, 1),
        Among(u"e", -1, 2),
        Among(u"o", -1, 1),
        Among(u"u", -1, 1),
        Among(u"à", -1, 1),
        Among(u"á", -1, 1),
        Among(u"â", -1, 1),
        Among(u"ä", -1, 1),
        Among(u"è", -1, 2),
        Among(u"é", -1, 2),
        Among(u"ê", -1, 2),
        Among(u"eë", -1, 3),
        Among(u"ië", -1, 4),
        Among(u"ò", -1, 1),
        Among(u"ó", -1, 1),
        Among(u"ô", -1, 1),
        Among(u"ö", -1, 1),
        Among(u"ù", -1, 1),
        Among(u"ú", -1, 1),
        Among(u"û", -1, 1),
        Among(u"ü", -1, 1)
    ]

    a_1 = [
        Among(u"nde", -1, 8),
        Among(u"en", -1, 7),
        Among(u"s", -1, 2),
        Among(u"'s", 2, 1),
        Among(u"es", 2, 4),
        Among(u"ies", 4, 3),
        Among(u"aus", 2, 6),
        Among(u"és", 2, 5)
    ]

    a_2 = [
        Among(u"de", -1, 5),
        Among(u"ge", -1, 2),
        Among(u"ische", -1, 4),
        Among(u"je", -1, 1),
        Among(u"lijke", -1, 3),
        Among(u"le", -1, 9),
        Among(u"ene", -1, 10),
        Among(u"re", -1, 8),
        Among(u"se", -1, 7),
        Among(u"te", -1, 6),
        Among(u"ieve", -1, 11)
    ]

    a_3 = [
        Among(u"heid", -1, 3),
        Among(u"fie", -1, 7),
        Among(u"gie", -1, 8),
        Among(u"atie", -1, 1),
        Among(u"isme", -1, 5),
        Among(u"ing", -1, 5),
        Among(u"arij", -1, 6),
        Among(u"erij", -1, 5),
        Among(u"sel", -1, 3),
        Among(u"rder", -1, 4),
        Among(u"ster", -1, 3),
        Among(u"iteit", -1, 2),
        Among(u"dst", -1, 10),
        Among(u"tst", -1, 9)
    ]

    a_4 = [
        Among(u"end", -1, 9),
        Among(u"atief", -1, 2),
        Among(u"erig", -1, 9),
        Among(u"achtig", -1, 3),
        Among(u"ioneel", -1, 1),
        Among(u"baar", -1, 3),
        Among(u"laar", -1, 5),
        Among(u"naar", -1, 4),
        Among(u"raar", -1, 6),
        Among(u"eriger", -1, 9),
        Among(u"achtiger", -1, 3),
        Among(u"lijker", -1, 8),
        Among(u"tant", -1, 7),
        Among(u"erigst", -1, 9),
        Among(u"achtigst", -1, 3),
        Among(u"lijkst", -1, 8)
    ]

    a_5 = [
        Among(u"ig", -1, 1),
        Among(u"iger", -1, 1),
        Among(u"igst", -1, 1)
    ]

    a_6 = [
        Among(u"ft", -1, 2),
        Among(u"kt", -1, 1),
        Among(u"pt", -1, 3)
    ]

    a_7 = [
        Among(u"bb", -1, 1),
        Among(u"cc", -1, 2),
        Among(u"dd", -1, 3),
        Among(u"ff", -1, 4),
        Among(u"gg", -1, 5),
        Among(u"hh", -1, 6),
        Among(u"jj", -1, 7),
        Among(u"kk", -1, 8),
        Among(u"ll", -1, 9),
        Among(u"mm", -1, 10),
        Among(u"nn", -1, 11),
        Among(u"pp", -1, 12),
        Among(u"qq", -1, 13),
        Among(u"rr", -1, 14),
        Among(u"ss", -1, 15),
        Among(u"tt", -1, 16),
        Among(u"v", -1, 4),
        Among(u"vv", 16, 17),
        Among(u"ww", -1, 18),
        Among(u"xx", -1, 19),
        Among(u"z", -1, 15),
        Among(u"zz", 20, 20)
    ]

    a_8 = [
        Among(u"d", -1, 1),
        Among(u"t", -1, 2)
    ]

    a_9 = [
        Among(u"", -1, -1),
        Among(u"eft", 0, 1),
        Among(u"vaa", 0, 1),
        Among(u"val", 0, 1),
        Among(u"vali", 3, -1),
        Among(u"vare", 0, 1)
    ]

    a_10 = [
        Among(u"ë", -1, 1),
        Among(u"ï", -1, 2)
    ]

    a_11 = [
        Among(u"ë", -1, 1),
        Among(u"ï", -1, 2)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass


class lab4(BaseException): pass


class lab5(BaseException): pass


class lab6(BaseException): pass


class lab7(BaseException): pass


class lab8(BaseException): pass


class lab9(BaseException): pass


class lab10(BaseException): pass
